# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_Msummary(NysolMOD_CORE):

	_kwd,_inkwd,_outkwd  = n_core.getparalist("msummary",3)

	def __init__(self,*args, **kw_args) :
		super(Nysol_Msummary,self).__init__("msummary",nutil.args2dict(args,kw_args,Nysol_Msummary._kwd))

def msummary(self,*args, **kw_args):
	return Nysol_Msummary(nutil.args2dict(args,kw_args,Nysol_Msummary._kwd)).addPre(self)

setattr(NysolMOD_CORE, "msummary", msummary)



# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_Mnormalize(NysolMOD_CORE):

	_kwd ,_inkwd,_outkwd  = n_core.getparalist("mnormalize",3)

	def __init__(self,*args, **kw_args) :
		super(Nysol_Mnormalize,self).__init__("mnormalize",nutil.args2dict(args,kw_args,Nysol_Mnormalize._kwd))


def mnormalize(self,*args, **kw_args):
	return Nysol_Mnormalize(nutil.args2dict(args,kw_args,Nysol_Mnormalize._kwd)).addPre(self)

setattr(NysolMOD_CORE, "mnormalize", mnormalize)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import os.path
import shutil

import nysol.mcmd as nm
import nysol.util as nu


class mgdiff(object):

	helpMSG="""
----------------------------
#{$cmd} version #{$version}
----------------------------
概要) ２つのグラフを比較し、同じ枝と異なる枝の情報を出力する。
書式) #{$cmd} ei= [ef=] eI= [eF=] [eo=] [no=] [T=] [--help]

  ファイル名指定
  ei=  : 入力枝ファイル1
  ef=  : 枝を構成するの2つの節点項目名(ei=上の項目名,デフォルト:node1,node2)
  eI=  : 入力枝ファイル2
  eF=  : eI=上の2つの節点項目名(ef=と同じであれば省略できる,デフォルト:ef=で指定した項目名)

  ni=  : 入力節点ファイル1(ei=に対応)
  nf=  : 枝を構成するの2つの節点項目名(ni=上の項目名,デフォルト:node)
  nI=  : 入力節点ファイル2(eI=に対応)
  nF=  : nI=上の2つの節点項目名(nf=と同じであれば省略できる,デフォルト:nf=で指定した項目名)

  eo=  : 出力枝ファイル
  no=  : 出力節点ファイル

	-dir : 有向グラフとして扱う

  その他
  T= : ワークディレクトリ(default:/tmp)
  --help : ヘルプの表示

  注1) 無向グラフとして扱う場合(デフォルト)、ei=ファイルとeI=ファイルとで、
       節点の並びが異なっていても、それは同じと見なす(ex. 枝a,bと枝b,aは同じ)。
       -dirを指定すれば異なるものと見なす。
  注2) 無向グラフとして扱う場合(デフォルト)、
       処理効率を重視し、ef=で指定した節点の並びはアルファベット順に並べ替えるため、
       eo=の項目の並びがei=やeI=の並びと異なることがある。
  注3) 同じ枝が複数ある場合、それらは単一化される。

入力データ)
ei=,eI=: 節点ペアからなるCSVファイル。
ni=,nI=: 節点からなるCSVファイル。

枝出力データ)
枝ファイル1と枝ファイル2のいずれかに出現する枝(節点ペア)について以下の値を出力する。
項目名: 内容
 ei    : ei=で指定したグラフにその行の節点ペアがあれば、そのファイル名
 eI    : eI=で指定したグラフにその行の節点ペアがあれば、そのファイル名
 diff : 差分の区分
         1: ei=のグラフにしか存在しない
         0: ei=,eI=の両方に存在する
        -1: eI=のグラフにしか存在しない

節点出力データ)
節点ファイル1と節点ファイル2のいずれかに出現する節点について以下の値を出力する。
項目名: 内容
 ni    : ni=で指定したグラフにその節点があれば、そのファイル名
 nI    : nI=で指定したグラフにその節点があれば、そのファイル名
 diff : 差分の区分
         1: ni=のグラフにしか存在しない
         0: ni=,nI=の両方に存在する
        -1: nI=のグラフにしか存在しない


例)
$ cat g1.csv
node1,node2
a,b
b,c
c,d

$ cat g2.csv
node1,node2
b,a
c,d
d,e

$ mgdiff.py ei=g1.csv eI=g2.csv ef=node1,node2
node1,node2,i,I,diff
a,b,g1.csv,g2.csv,0
b,c,g1.csv,,1
c,d,g1.csv,g2.csv,0
d,e,,g2.csv,-1

$ mgdiff.py ei=g1.csv eI=g2.csv ef=node1,node2 -dir
node1,node2,i,I,diff
a,b,data/g1.csv,,1
b,a,,data/g2.csv,-1
b,c,data/g1.csv,,1
c,d,data/g1.csv,data/g2.csv,0
d,e,,data/g2.csv,-1

# Copyright(c) NYSOL 2012- All Rights Reserved.
	"""

	verInfo="version=1.2"

	paramter = {	
		"ei":"filename",
		"ef":"fld",
		"eI":"filename",
		"eF":"fld",
		"ni":"filename",
		"nf":"fld",
		"nI":"filename",
		"nF":"fld",
		"eo":"filename",		
		"no":"filename",
		"dir":"bool",
		"T":"str"
	}

	paramcond = {	
		"hissu": ["ei","eI","eo"]
	}

	def help():
		print(mgdiff.helpMSG) 

	def ver():
		print(mgdiff.versionInfo)



	def __param_check_set(self , kwd):

		for k,v in kwd.items():
			if not k in mgdiff.paramter	:
				raise( Exception("KeyError: {} in {} ".format(k,self.__class__.__name__) ) )

		self.msgoff = True

		# edgeファイル名
		if not "ei" in kwd :
			raise Exception("ei= is necessary")
		self.ei = kwd["ei"] 
		
		# edgeファイル名
		if not "eI" in kwd :
			raise Exception("eI= is necessary")
		self.eI = kwd["eI"] 
		

		self.ni = kwd["ni"] if "ni" in kwd else None # node file name
		self.nI = kwd["nI"] if "nI" in kwd else None # node file name

		if (self.ni and self.nI==None) or (self.ni == None and self.nI) :
			raise("must specify both of ni= and nI=")


		# edge出力ファイル名
		if not "eo" in kwd :
			raise Exception("eo= is necessary")
		self.eo = kwd["eo"] 

		# Node出力ファイル名
		self.no = kwd["no"] if "no" in kwd else None # node file name

		# ---- edge field names (two nodes) on ei=
		if "ef" in kwd:
			ef0 = kwd["ef"].split(",")
			self.ef1 = ef0[0]
			self.ef2 = ef0[1] 
		else:
			self.ef1 = "node1"
			self.ef2 = "node2"

		if "eF" in kwd:
			ef0 = kwd["eF"].split(",")
			self.eF1 = ef0[0]
			self.eF2 = ef0[1] 
		else:
			self.eF1 = self.ef1 
			self.eF2 = self.ef2 

		# ---- node field name on ni=
		self.nf = kwd["nf"] if "nf"  in kwd else "node"

		self.nF = kwd["nF"] if "nF"  in kwd else "node"

		if not "eF" in kwd: #なぜ？
			self.nF = self.nf

		self.dir = kwd["dir"] if "dir" in kwd else False

		#ワークファイルパス
		if "T" in kwd :
			os.environ["KG_TmpPath"] =  re.sub('\/$','',kwd["T"])

	def __cmdline(self):
		cmdline = self.__class__.__name__
		for k,v in self.args.items():
			if type(v) is bool :
				if v == True :
					cmdline += " -" + str(k)
			else:
				cmdline += " " + str(k) + "=" + str(v)
		return cmdline 

	def __init__(self,**kwd):

		#パラメータチェック
		self.args = kwd
		self.__param_check_set(kwd)

	# ============
	# entry point
	def run(self,**kw_args):
	
		os.environ['KG_ScpVerboseLevel'] = "2"
		if "msg" in kw_args:
			if kw_args["msg"] == "on":
				os.environ['KG_ScpVerboseLevel'] = "4"

		from datetime import datetime	
		t = datetime.now()

		wf = nu.Mtemp()
		xxedge_i = wf.file()
		xxedge_I = wf.file()

		# クリーニング(ei=)
		fei = None
		fei <<= nm.mcut(f=[self.ef1,self.ef2], i=self.ei)
		if not self.dir :
			fei <<= nm.mfsort(f=[self.ef1,self.ef2])
		fei <<= nm.muniq(k=[self.ef1,self.ef2])
		fei <<= nm.msetstr(a="ei",v=self.ei)
		
		# クリーニング(ei=)
		feI = None
		feI <<= nm.mcut(f="%s:%s,%s:%s"%(self.eF1,self.ef1,self.eF2,self.ef2), i=self.ei)
		if not self.dir :
			feI <<= nm.mfsort(f=[self.ef1,self.ef2])
		feI <<= nm.muniq(k=[self.ef1,self.ef2])
		feI <<= nm.msetstr(a="eI",v=self.eI)

		fe = None
		fe <<= nm.mjoin(k=[self.ef1,self.ef2], m=feI,i=fei,f="eI",n=True,N=True)
		fe <<= nm.mcal(c='if(isnull($s{ei}),-1,if(isnull($s{eI}),1,0))',a="diff")
		fe <<= nm.mcut(f=[self.ef1,self.ef2,"ei","eI","diff"],o=self.eo)

		fe.run()

		if self.ni and self.nI :
			# クリーニング(ni=)
			fni =   nm.mcut(f=self.nf,i=self.ni)
			fni <<= nm.muniq(k=self.nf)
			fni <<= nm.msetstr(a="ni",v=self.ni)

			fnI =   nm.mcut(f="%s:%s"%(self.nF,self.nf),i=self.nI)
			fnI <<= nm.muniq(k=self.nf)
			fnI <<= nm.msetstr(a="nI",v=self.nI)
			
			fn = None
			fn <<= nm.mjoin(k=self.nf,m=fnI,i=fni,f="nI",n=True,N=True)
			fn <<= nm.mcal(c='if(isnull($s{ni}),-1,if(isnull($s{nI}),1,0))',a="diff")
			fn <<= nm.mcut(f=[self.nf,"ni","nI","diff"],o=self.no)

			fn.run()



		nu.mmsg.endLog(self.__cmdline())






from .nysollib.core import runs,drawModels,drawModelsD3,makeScps,modelInfos,setMsgFlg,setRunLimit
from .submod.readcsv import Nysol_Readcsv as readcsv
from .submod.readlist import Nysol_Readlist as readlist
from .submod.list2csv import Nysol_List2Csv as list2csv
from .submod.writecsv import Nysol_Writecsv as writecsv
from .submod.writelist import Nysol_Writelist as writelist
from .submod.writedict import Nysol_Writedict as writedict
from .submod.mfifo import Nysol_Mfifo as mfifo
from .submod.m2cross import Nysol_M2cross as m2cross
from .submod.maccum import Nysol_Maccum as maccum
from .submod.mavg import Nysol_Mavg as mavg
from .submod.mbest import Nysol_Mbest as mbest
from .submod.mbucket import Nysol_Mbucket as mbucket
from .submod.mchgnum import Nysol_Mchgnum as mchgnum
from .submod.mchgstr import Nysol_Mchgstr as mchgstr
from .submod.mcombi import Nysol_Mcombi as mcombi
from .submod.mcommon import Nysol_Mcommon as mcommon
from .submod.mcount import Nysol_Mcount as mcount
from .submod.mcross import Nysol_Mcross as mcross
from .submod.mdelnull import Nysol_Mdelnull as mdelnull
from .submod.mdformat import Nysol_Mdformat as mdformat
from .submod.mduprec import Nysol_Mduprec as mduprec
from .submod.mfldname import Nysol_Mfldname as mfldname
from .submod.mfsort import Nysol_Mfsort as mfsort
from .submod.mhashavg import Nysol_Mhashavg as mhashavg
from .submod.mhashsum import Nysol_Mhashsum as mhashsum
from .submod.mkeybreak import Nysol_Mkeybreak as mkeybreak
from .submod.mmbucket import Nysol_Mmbucket as mmbucket
from .submod.mmvavg import Nysol_Mmvavg as mmvavg
from .submod.mmvsim import Nysol_Mmvsim as mmvsim
from .submod.mmvstats import Nysol_Mmvstats as mmvstats
from .submod.mnewnumber import Nysol_Mnewnumber as mnewnumber
from .submod.mnewrand import Nysol_Mnewrand as mnewrand
from .submod.mnewstr import Nysol_Mnewstr as mnewstr
from .submod.mnjoin import Nysol_Mnjoin as mnjoin
from .submod.mnormalize import Nysol_Mnormalize as mnormalize
from .submod.mnrcommon import Nysol_Mnrcommon as mnrcommon
from .submod.mnrjoin import Nysol_Mnrjoin as mnrjoin
from .submod.mnullto import Nysol_Mnullto as mnullto
from .submod.mnumber import Nysol_Mnumber as mnumber
from .submod.mpadding import Nysol_Mpadding as mpadding
from .submod.mpaste import Nysol_Mpaste as mpaste
from .submod.mproduct import Nysol_Mproduct as mproduct
from .submod.mrand import Nysol_Mrand as mrand
from .submod.mrjoin import Nysol_Mrjoin as mrjoin
from .submod.msed import Nysol_Msed as msed
from .submod.msel import Nysol_Msel as msel
from .submod.mselnum import Nysol_Mselnum as mselnum
from .submod.mselrand import Nysol_Mselrand as mselrand
from .submod.mselstr import Nysol_Mselstr as mselstr
from .submod.msetstr import Nysol_Msetstr as msetstr
from .submod.mshare import Nysol_Mshare as mshare
from .submod.msim import Nysol_Msim as msim
from .submod.mslide import Nysol_Mslide as mslide
from .submod.msplit import Nysol_Msplit as msplit
from .submod.mstats import Nysol_Mstats as mstats
from .submod.msummary import Nysol_Msummary as msummary
from .submod.mtonull import Nysol_Mtonull as mtonull
from .submod.mtra import Nysol_Mtra as mtra
from .submod.mtraflg import Nysol_Mtraflg as mtraflg
from .submod.muniq import Nysol_Muniq as muniq
from .submod.mvcat import Nysol_Mvcat as mvcat
from .submod.mvcommon import Nysol_Mvcommon as mvcommon
from .submod.mvcount import Nysol_Mvcount as mvcount
from .submod.mvdelim import Nysol_Mvdelim as mvdelim
from .submod.mvdelnull import Nysol_Mvdelnull as mvdelnull
from .submod.mvjoin import Nysol_Mvjoin as mvjoin
from .submod.mvnullto import Nysol_Mvnullto as mvnullto
from .submod.mvreplace import Nysol_Mvreplace as mvreplace
from .submod.mvsort import Nysol_Mvsort as mvsort
from .submod.mvuniq import Nysol_Mvuniq as mvuniq
from .submod.mwindow import Nysol_Mwindow as mwindow
from .submod.mtrafld import Nysol_Mtrafld as mtrafld
from .submod.mcal import Nysol_Mcal as mcal
from .submod.mcut import Nysol_Mcut as mcut
from .submod.mcat import Nysol_Mcat as mcat
from .submod.msum import Nysol_Msum as msum
from .submod.msep import Nysol_Msep as msep
from .submod.mshuffle import Nysol_Mshuffle as mshuffle
from .submod.cmd import Nysol_Excmd as cmd
from .submod.runfunc import Nysol_Runfunc as runfunc
from .submod.marff2csv import Nysol_Marff2csv as marff2csv
from .submod.m2tee import Nysol_M2tee as m2tee
from .submod.mtab2csv import Nysol_Mtab2csv as mtab2csv
from .submod.mxml2csv import Nysol_Mxml2csv as mxml2csv
from .submod.msortf import Nysol_Msortf as msortf
from .submod.mjoin import Nysol_Mjoin as mjoin
from .submod.mkmeans import Nysol_Mkmeans as mkmeans
from .submod.m2tee import Nysol_M2tee as m2tee
from .submod.m2cat import Nysol_M2cat as m2cat
from .submod.municat import Nysol_Municat as municat
from .submod.mstdin import Nysol_Mstdin as mstdin
from .submod.mstdout import Nysol_Mstdout as mstdout
from .submod.mread import Nysol_Mread as mread
from .submod.mwrite import Nysol_Mwrite as mwrite




# -*- coding: utf-8 -*-
#/* ////////// LICENSE INFO ////////////////////
#
# * Copyright (C) 2013 by NYSOL CORPORATION
# *
# * Unless you have received this program directly from NYSOL pursuant
# * to the terms of a commercial license agreement with NYSOL, then
# * this program is licensed to you under the terms of the GNU Affero General
# * Public License (AGPL) as published by the Free Software Foundation,
# * either version 3 of the License, or (at your option) any later version.
# * 
# * This program is distributed in the hope that it will be useful, but
# * WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF 
# * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
# *
# * Please refer to the AGPL (http://www.gnu.org/licenses/agpl-3.0.txt)
# * for more details.
#
# ////////// LICENSE INFO ////////////////////*/

import os, sys

def dsplen(data):
				
	if sys.version_info.major < 3:
		datax = data.decode('utf-8')
	else:
		datax = data

	fsize=0
	for charstr in datax:
		if ord(charstr)<128 :
			fsize +=1
		else:
			fsize +=2
	return fsize

def dspchg(data,size):

	if sys.version_info.major < 3:
		datax = data.decode('utf-8')
	else:
		datax = data

	fsize=0

	for i , charstr in enumerate(datax):

		if ord(charstr)<128 :
			fsize +=1
		else:
			fsize +=2

		if fsize > size-3:
			return data[0:i]+"..."

	return data

def sizeCHK(data,premax):
	
	for lin in data:

		for i, fdata in enumerate(lin):

			fsize = dsplen(fdata)
			if fsize > premax[i] :
				premax[i] = fsize


def chgDSPstr(pre,dmy=False,fldnameLimit=15):

	try:
		if hasattr(os,"get_terminal_size"):
			width = os.get_terminal_size().columns
		else:
			_,width_str = os.popen('stty size').read().split()
			width = int(width_str)
	except:
		width=80

	sufmax = int(len(pre)/2)

	# fldcut check
	fldmax = [ 0 for i in range(len(pre[0]))]
	sizeCHK(pre,fldmax)

	for i in range(len(fldmax)):
		if fldmax[i] > fldnameLimit:
			fldmax[i] = fldnameLimit

	fldcut=False
	if sum(fldmax) +len(fldmax) > width:

		dspfldNo =[]
		restW = width - (fldmax[-1]+5)

		for i ,v in enumerate(fldmax):

			if restW - v > 0 :
				dspfldNo.append(i) 
			else:
				dspfldNo.append(len(fldmax)-1)
				break 

			restW -= (v+1)

		fldcut = True

	else:
		
		dspfldNo =  [i for i in range(len(fldmax))]

	# make output string
	outstr=[]
	for i,lin in enumerate(pre):

		# dmy行
		if dmy and i == sufmax :

			dmystr = []

			for pos in dspfldNo:

				if fldcut and pos == len(fldmax)-1:
					dmystr.append("...")   

				dmylen = fldmax[pos] if fldmax[pos]<3 else 3
				fmt = "%%%ds"%(fldmax[pos])
				dmystr.append(fmt%("."*dmylen))
							
			outstr.append(" ".join(dmystr))


		newstr=[]

		for pos in dspfldNo:

			if fldcut and pos == len(fldmax)-1:
				newstr.append("...")   

			dlen = dsplen(lin[pos])

			if dlen <= fldmax[pos] :

				fmtdlen = (fldmax[pos]-dlen)+len(lin[pos])
				fmt = "%%%ds"%(fmtdlen)
				newstr.append(fmt%(lin[pos]))
			
			else:

				newdata = dspchg(lin[pos],fldmax[pos])
				fmtdlen = (fldmax[pos]-dsplen(newdata))+len(newdata)
				fmt = "%%%ds"%(fmtdlen)
				newstr.append(fmt%(newdata))

		outstr.append(" ".join(newstr))

	return outstr



def chgDSPhtml(dspdata, yLimit , maxno,head):

		mid = yLimit / 2
		arange_number = 0

		dmy = ( maxno > yLimit )

		outstrList=[]
		outstrList.append("<div>")
		outstrList.append("<table border=\"1\">")
		outstrList.append("<thead>")
		if head != None:
			outstrList.append("<tr>")
			outstrList.append("<th></th>")
			for v in head:
				outstrList.append("<th>{}</th>".format(v))
			outstrList.append("</tr>")

		outstrList.append("</thead>")
		outstrList.append("<tbody>")

		for i,ldata in enumerate(dspdata):

			if i == mid and dmy: 
				arange_number = maxno - yLimit
				outstrList.append("<tr>")
				outstrList.append("<th>...</th>")
				for v in ldata:
					outstrList.append("<td>...</td>")
				outstrList.append("</tr>")


			outstrList.append("<tr>")
			outstrList.append("<th>{}</th>".format(i+arange_number))

			for v in ldata:
				outstrList.append("<td>{}</td>".format(v))
			outstrList.append("</tr>")

		outstrList.append("</tbody>")
		outstrList.append("</table>")
		outstrList.append("</div>")

		return outstrList





#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import nysol.util.margs as margs
import nysol.take as nt


args=margs.Margs(sys.argv,"i=,x=,O=,tid=,item=,cls=,taxo=,type=,s=,S=,sx=,Sx=,g=,p=,-uniform,l=,u=,top=,T=,-replaceTaxo")
nt.mitemset(**(args.kvmap())).run(msg="on")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import nysol.mcmd as nm
import nysol.util as nu

from nysol.take.lib.base import seqDB as sdb
from nysol.take.lib import taxonomy as taxolib
from nysol.take.lib import enumLcmSeq as elcmSeq
from nysol.take.lib import enumLcmEsp as elcmEsp
from nysol.take import extcore as extTake


class msequence(object):
	def helpMSG(self):
		msg="""
------------------------
#{$cmd} version #{$version}
------------------------
概要) LCM_seqにより多頻度系列パターンを列挙する
特徴) 1) 
書式) #{$cmd} i= [x=] [O=] [tid=] [item=] [time=] [taxo=] [class=] [s=] [S=] [sx=] [Sx=] [l=] [u=] [top=] [p=] [g=] [w=] [Z=] [T=] [--help]

例1) #{$cmd} i=weblog.csv c=customer.csv tid=traID item=page time=アクセス時刻 class=購買
例2) #{$cmd} i=weblog.csv tid=traID item=page time=アクセス時刻

  ファイル名指定
  i= : アイテム集合データベースファイル名【必須】
  c= : クラスファイル名【オブション】
  x= : taxonomyファイル名【オブション】*1
  O= : 出力ディレクトリ名【オプション:default:./take_現在日付時刻】

  項目名指定
  tid=   : トランザクションID項目名(i=,c=上の項目名)【オプション:default="tid"】
  time=  : 時間項目名(i=上の項目名)【オプション:default="time"】
  item=  : アイテム項目名(i=,t=上の項目名)【オプション:default="item"】
  class= : クラス項目名(c=上の項目名)【オプション:default="class"】
  taxo=  : 分類項目名を指定する【条件付き必須:x=】

  列挙パラメータ
  s=   : 最小支持度(support)【オプション:default:0.05, 0以上1以下の実数】
  S=   : 最小支持件数【オプション】
  sx=  : 最大支持度(support)【オプション:default:1.0, 0以上1以下の実数】
  Sx=  : 最大支持件数【オプション】
  l=   : パターンサイズの下限(1以上20以下の整数)【オプション:default:制限なし】
  u=   : パターンサイズの上限(1以上20以下の整数)【オプション:default:制限なし】
  p=   : 最小事後確率【オプション:default:0.5】
  g=   : 最小増加率【オプション】
  gap= : パターンのギャップ長の上限(0以上の整数)【オプション:0で制限無し,default:0】
  win= : パターンのウィンドウサイズの上限(0以上の整数)【オプション:0で制限無し,default:0】
  -padding : 時刻を整数とみなし、連続でない時刻に特殊なアイテムがあることを想定する。
           : gap=やwin=の指定に影響する。
  top=  : 列挙するパターン数の上限【オプション:default:制限なし】*2

  その他
  T= : ワークディレクトリ(default:/tmp)
  --help : ヘルプの表示

  注釈
  *1 x=が指定されたとき、item項目の値を対応するtaxonomyに変換して実行する。例えば、アイテムa,bのtaxonomyをX、c,dのtaxonomyをYとすると、
     シーケンス"aeadd"は"XeXYY"に変換される。
  *2 top=が指定された時の動作: 例えばtop=10と指定すると、支持度が10番目高いパターンの支持度を最小支持度として頻出パターンを列挙する。よって、同じ支持度のパターンが複数個ある場合は10個以上のパターンが列挙されるかもしれない。

# より詳しい情報源 http://www.nysol.jp
# LCM_seqの詳しい情報源 http://research.nii.ac.jp/~uno/codes-j.htm
# Copyright(c) NYSOL 2012- All Rights Reserved.
		"""

	verInfo="version=1.2"


	paramter = {	
		"i": "filename",
		"c": "fileName", # なにもしてない
		"x": "fileName",
		"O": "str",
		"tid": "fld",
		"item":"fld",
		"time":"fld",
		"cls" :"fld",
		"taxo":"fld",
		"s" : "float",
		"S":"int",
		"sx":"float",
		"Sx":"int",
		"g":"float",
		"p": "float",
		"uniform" : "bool",
		"l":"int",
		"u":"int",
		"gap":"int",
		"win":"int",
		"top":"int",
		"padding" : "bool",
		"mcmdenv" : "bool",
		"T": "str"
	}
	paramcond = {	
		"hissu":"i"
	}

	def help():
		print(msequence.helpMSG) 

	def ver():
		print(msequence.versionInfo)


	def __param_check_set(self , kwd):

		for k,v in kwd.items():
			if not k in msequence.paramter	:
				raise( Exception("KeyError: {} in {} ".format(k,self.__class__.__name__) ) )
			#型チェック入れる

		import datetime
		t = datetime.datetime.today()
		self.iFile   = kwd["i"]
		self.xFile   = kwd["x"]    if "x"    in kwd else None
		self.idFN    = kwd["tid"]  if "tid"  in kwd else "tid"
		self.timeFN  = kwd["time"] if "time" in kwd else "time"
		val2=self.timeFN.split("%")
		self.timeFN = val2[0]

		self.itemFN  = kwd["item"] if "item" in kwd else "item"
		self.clsFN   = kwd["cls"]  if "cls"  in kwd else None


		self.taxoFN  = kwd["taxo"] if "taxo" in kwd else "taxo"
		self.outPath = kwd["O"]    if "O"    in kwd else "./take_{}".format(t.strftime("%Y%m%d%H%M%S"))

		self.eArgs={}
		self.eArgs["minSup" ] = float(kwd["s"]) if "s" in kwd else 0.05  # 0-1
		self.eArgs["minProb"] = float(kwd["p"]) if "p" in kwd else 0.5   # 0.5-1
		self.eArgs["uniform"] = kwd["uniform"] if "uniform" in kwd else False
		self.eArgs["padding"] = kwd["padding"] if "padding" in kwd else False

		if "S" in kwd :
			self.eArgs["minCnt"] = int(kwd["S"])

		if "sx" in kwd :
			self.eArgs["maxSup"] = float(kwd["sx"]) # 0-1

		if "Sx" in kwd :
			self.eArgs["maxCnt"] = int(kwd["Sx"]) # 1-

		if "g" in kwd :
			self.eArgs["minGR"] = float(kwd["g"]) # 1.0

		if "l" in kwd :
			self.eArgs["minLen"] = int(kwd["l"]) # 1-

		if "u" in kwd :
			self.eArgs["maxLen"] = int(kwd["u"]) # 1-

		if "gap" in kwd :
			self.eArgs["gap"] = int(kwd["gap"]) # 0-

		if "win" in kwd :
			self.eArgs["win"] = int(kwd["win"]) # 0-

		if "top" in kwd :
			self.eArgs["top"] = int(kwd["top"])

		if "minLen" in self.eArgs and "maxLen" in self.eArgs:
			if self.eArgs["minLen"] > self.eArgs["maxLen"]:
				raise Exception("u= must be greater than or equal to l=")

		if "gap" in self.eArgs and "win" in self.eArgs:
			if self.eArgs["gap"] > self.eArgs["win"] :
				raise Exception("win= must be greater than or equal to gap=")

	def __cmdline(self):
		cmdline = self.__class__.__name__
		for k,v in self.args.items():
			if type(v) is bool :
				if v == True :
					cmdline += " -" + str(k)
			else:
				cmdline += " " + str(k) + "=" + str(v)
		return cmdline 


	def __init__(self,**kwd):
		#パラメータチェック
		self.args = kwd
		self.__param_check_set(kwd)


	def run(self,**kw_args):

		os.environ['KG_ScpVerboseLevel'] = "2"
		if "msg" in kw_args:
			if kw_args["msg"] == "on":
				os.environ['KG_ScpVerboseLevel'] = "4"

		# seq型DBの読み込み
		db=sdb.SeqDB(self.iFile,self.idFN,self.timeFN,self.itemFN,self.eArgs["padding"],self.clsFN)
		#db.show

		# taxonomyのセット
		taxo=None
		if self.xFile!=None :
			self.taxo = taxolib.Taxonomy(self.xFile,self.itemFN,self.taxoFN)
			db.repTaxo(taxo) # seqはtaxonomyの置換のみ

		# パターン列挙
		lcm=None
		if self.clsFN:
			lcm=elcmEsp.LcmEsp(db);
			lcm.enumerate(self.eArgs)
		else:
			lcm=elcmSeq.LcmSeq(db);
			lcm.enumerate(self.eArgs)

		# 出力
		if not os.path.isdir(self.outPath) :
			os.makedirs(self.outPath)


		lcm.output(self.outPath)


		nu.mmsg.endLog(self.__cmdline())



# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_Mshuffle(NysolMOD_CORE):

	_kwd,_inkwd,_outkwd  = n_core.getparalist("mshuffle",3)
	_disabled_ouputlist = True

	def __init__(self,*args, **kw_args) :
		super(Nysol_Mshuffle,self).__init__("mshuffle",nutil.args2dict(args,kw_args,Nysol_Mshuffle._kwd))

def mshuffle(self,*args, **kw_args):
	return Nysol_Mshuffle(nutil.args2dict(args,kw_args,Nysol_Mshuffle._kwd)).addPre(self)

setattr(NysolMOD_CORE, "mshuffle", mshuffle)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#/* ////////// LICENSE INFO ////////////////////
#
# * Copyright (C) 2013 by NYSOL CORPORATION
# *
# * Unless you have received this program directly from NYSOL pursuant
# * to the terms of a commercial license agreement with NYSOL, then
# * this program is licensed to you under the terms of the GNU Affero General
# * Public License (AGPL) as published by the Free Software Foundation,
# * either version 3 of the License, or (at your option) any later version.
# * 
# * This program is distributed in the hope that it will be useful, but
# * WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF 
# * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
# *
# * Please refer to the AGPL (http://www.gnu.org/licenses/agpl-3.0.txt)
# * for more details.
#
# ////////// LICENSE INFO ////////////////////*/
import re

def margv2dict(argv, klist ,mandatory=None,conv=None):
	
	if conv == None:
		conv = {} 

	if isinstance(klist[0],str) :
		keyPair   = set(klist[0].split(','))
	elif isinstance(klist[0],list) :
		keyPair   = set(klist[0])
	else:
		raise TypeError("keyilst type error")

	if isinstance(klist[1],str) :
		keySingle = set(klist[1].split(','))
	elif isinstance(klist[1],list) :
		keySingle = set(klist[1])
	else:
		raise TypeError("keyilst type error")

	kw_args ={}
	for arg in argv[1:] :
		val = arg.split("=",1)

		if len(val)==1 :
			if val[0][0] == '-':
				valm = re.sub(r'^-', "", val[0])
				if valm in keySingle:
					if valm in conv:
						kw_args[conv[valm]] = True
					else:
						kw_args[valm] = True
			else:
				raise TypeError("unkonwn parameter "+ val)

		elif len(val)==2 :
			if val[0] in keyPair:
				if val[0] in conv:
						kw_args[conv[val[0]]] = val[1]
				else:
						kw_args[val[0]] = val[1]
			else:	
				raise TypeError("unkonwn parameter "+ val[0])

	mlist =None
	if mandatory == None:
		return kw_args
	if isinstance(mandatory,str) :
		mlist = mandatory.split(',')
	elif isinstance(mandatory,list) :
		mlist = mandatory
	else:	
		raise TypeError("mandatory type error")


	for mkey in mlist :
		if not mkey in kw_args:
			raise ValueError(mkey + "is mandatory ")
	
	return kw_args



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import re
import nysol.util.margs as margs
import nysol.take.graph as ntg
import nysol.take as nt

args=margs.Margs(sys.argv,"ni=,nf=,ei=,ef=,-indirect,eo=,no=,th=,sim=,sup=,iter=,log=,O=","ei=,ef=,th=")

ei = args.str("ei=")
ni = args.str("ni=",None)
ef = args.str("ef=",None,",")
nf = args.str("nf=",None)
if len(ef) != 2 :
	raise Exception("ef size is 2")

gi=ntg.graph(edgeFile=ei,title1=ef[0],title2=ef[1],nodeFile=ni,title=nf)
para={}
para["gi"]=gi

for k,v in args.keyValue.items():

	if k == "ei=" or k == "ef=" or k == "nf=" or k == "ni=" or k == "eo=" or k == "no=" :
		continue
	if re.match(r'^-',k):
		para[re.sub(r'^-',"",k)]=v
	else:	
		para[re.sub(r'=$',"",k)]=v

gobj = nt.mpolishing(**para).run(msg="on")

ofile_e =args.str("eo=")
ofile_n =args.str("no=")

if ofile_e:
	ro = gobj.edges()
	ro.writecsv(o=ofile_e).run()

if ofile_n:
	ro = gobj.nodes()
	ro.writecsv(o=ofile_n).run()


	




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

import shutil
#import nysol.util.mtemp as mtemp
#import nysol.util.mrecount as mrecount

import nysol.mcmd as nm
import nysol.util as nu

from nysol.take.lib import taxonomy as taxolib

#=アイテムクラス
# 頻出パターンマイニングで使われるアイテムを扱うクラス。
#
#===利用例
# 以下tra.csvの内容
# ---
# items
# a b c
# a c
# b
# c b d
#
# 以下taxo.csvの内容
# ---
# item,taxo
# a,X1
# b,X1
# c,X2
# d,X2
#
# 以下rubyスクリプト
# ---
# require 'rubygems'
# require 'mining'
# items=Items.new("tra.csv","items","item")
# puts items.file     # => ./1252810329_1850/dat/1
# puts items.itemFN   # => "item"
# puts items.idFN     # => "iid"
# puts items.size     # => 4
# puts items.taxonomy # =>nil
#
# taxo=Taxonomy.new("taxo.csv","item","taxo")
# items.addTaxo(taxo)
# puts items.file     # => ./1252810329_1850/dat/3
# puts items.itemFN   # => "item"
# puts items.idFN     # => "iid"
# puts items.size     # => 4
# p    items.taxonomy # => #<Taxonomy:0x1698eac @itemSize=4, @iFile="taxo.csv", @taxoFN="taxo", @itemFN="item", ...,@file="./1252810329_1850/dat/2">
#
#=====./1252810329_1850/dat/1 の内容
# item,iid
# a,1
# b,2
# c,3
# d,4
#
#=====./1252810329_1850/dat/3 の内容
# item,iid
# X1,5
# X2,6
# a,1
# b,2
# c,3
# d,4
#
class Items(object):

	#==初期化
	#=====引数
	# iFile: アイテム項目を含むファイル名
	# itemFN: 新しいアイテム項目名
	# idFN: 連番によるアイテムidの項目名(デフォルト:"iid")
	#=====機能
	#* iFile上のアイテム項目(@itemFN)からアイテム(@itemFN)と連番("num")の2項目からなるファイルを生成する。
	#* itemFN項目の値としては、スペースで区切られた複数のアイテムであってもよい。
	#* 同じアイテムが重複して登録されていれば単一化される。
	#* アイテム順にソートされる。
	#* アイテムの件数(種類数)がセットされる。
	def  __init__(self,iFile,itemFN,idFN="__iid"):
		self.file = None

		# アイテムの項目名(=>String)
		self.itemFN = None

		# id項目名(=>String)
		self.idFN = None

		# アイテムの種類数(=>Fixnum)
		self.size = None

		# 分類階層クラス(=>Taxonomy)
		self.taxonomy = None

		self.temp = nu.Mtemp()
		self.iFile   = []
		self.iPath   = []
		self.iFile.append(iFile)
		self.iPath.append(os.path.abspath(iFile))
		self.taxonomy = None
		self.file     = self.temp.file()  # 出力ファイル名
		self.itemFN   = itemFN
		self.idFN     = idFN

		nm.mcut(f=itemFN,i=iFile).muniq(k=itemFN).mnumber(s=itemFN,a=idFN,S=0,o=self.file).run()

		self.size = nu.mrecount(i=self.file) # itemの数


	def show(self):
		print("#### BEGIN Items class")
		#print("@temp=%s"%(self.temp))
		print("@iFile=%s"%(','.join(self.iFile)))
		print("@iPath=%s"%(','.join(self.iPath)))
		#print("@taxonomy=#{@taxonomy}"%(@temp))
		print("@itemFN=%s"%(self.itemFN))
		print("@idFN=%s"%(self.idFN))
		print("@file=%s"%(self.file ))
		print("@file:")
		os.system("cat "+self.file)
		print("#### END Items class")

	#==アイテムの追加
	# iFileのitemFN項目をアイテムとして追加する。
	#=====引数
	# iFile: アイテム項目を含むファイル名
	# itemFN: iFile上のアイテム項目名
	#=====機能
	#* itemFNとしては、スペースで区切られた複数のアイテムであってもよい。
	#* 同じアイテムが重複していれば単一化される。
	#* 追加後アイテム順にソートされる。
	#* アイテム数が更新される。
	def add(self,iFile,itemFN):

		self.iFile.append(iFile)
		self.iPath.append(os.path.abspath(iFile))

		xx=nu.Mtemp()
		xx1=xx.file()
		xx2=xx.file()
		x0 =   nm.mcommon(k=self.itemFN,i=iFile,m=self.file)
		x0 <<= nm.mcut(f=itemFN+":"+self.itemFN)
		x0 <<= nm.muniq(k=self.itemFN)
		x0 <<= nm.mnumber(s=self.itemFN,S=self.size+1,a=self.idFN,o=xx1)
		x0.run()

		nm.mcat(i=self.file+","+xx1).msortf(f=self.itemFN,o=xx2).run()

		# 新itemファイル登録&item数更新
		shutil.move(xx2,self.file)
		self.size = nu.mrecount(i=self.file)


	#==Taxonomyの設定
	# Taxonomy(分類階層)を設定する。
	#=====引数
	# taxo: taxonomy オブジェクト
	#=====機能
	#* Taxonomyオブジェクトをメンバとして追加する。
	#* 分類項目をアイテムとして追加する。
	def addTaxo(self,taxo):
		self.taxonomy=taxo
		self.add(self.taxonomy.file,self.taxonomy.taxoFN) # taxonomyをアイテムとして追加登録

	def repTaxo(self,taxo):

		#@taxonomy=taxo #replaceの場合はtaxonomyを登録しない
		self.file = self.temp.file() # 出力ファイル名
		nm.mcut(i=taxo.file,f=taxo.taxoFN+":"+self.itemFN).muniq(k=self.itemFN).mnumber(s=self.itemFN,a=self.idFN,S=1,o=self.file).run()
		




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#/* ////////// LICENSE INFO ////////////////////
#
# * Copyright (C) 2013 by NYSOL CORPORATION
# *
# * Unless you have received this program directly from NYSOL pursuant
# * to the terms of a commercial license agreement with NYSOL, then
# * this program is licensed to you under the terms of the GNU Affero General
# * Public License (AGPL) as published by the Free Software Foundation,
# * either version 3 of the License, or (at your option) any later version.
# * 
# * This program is distributed in the hope that it will be useful, but
# * WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF 
# * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
# *
# * Please refer to the AGPL (http://www.gnu.org/licenses/agpl-3.0.txt)
# * for more details.
#
# ////////// LICENSE INFO ////////////////////*/


import os
import copy

	
class MparallelManager(object):

	def __init__(self,mp=4):
		self.mp = mp 					# パラレルサイズ
		self.runpid = {} 			# pid => laneNo ## 動いてるPROCESS
		self.LaneQue = list(range(mp))

		# stsチェックするなら追加
		#self.thInterval = tim # チェック間隔
		#self.slppid = []			# [ [pid ,laneNo child pid] ... ## 休止中PROCESS
		#self.mtx =  Mutex.new if @thInterval > 0
		#self.mtx =  None


	# プロセス終了確認
	def waitLane(self):

		finLane =[]
		
		while(True):
			waitbrk=False
			rpid = None
			while(True):
				for k in self.runpid:
					if os.waitpid(k,os.WNOHANG) != (0,0):
						waitbrk = True
						rpid = k
						break
				if waitbrk:
					break

			new_pno = None
			if rpid != None:
				new_pno = self.runpid.pop(rpid)	

			if new_pno != None :
				finLane.append(new_pno)
				self.LaneQue.append(new_pno)
				break
				
		return finLane
		
	# 全プロセス終了確認
	def waitall(self):
		rtn = []
		while(len(self.runpid)!=0 or len(self.runpid)!=0):
			rtn.extend(self.waitLane()) 

		return rtn

	# 空き実行レーン取得
	def getLane(self , wait=True):
		if wait and len(self.LaneQue)==0:
			self.waitLane()
		return self.LaneQue.pop(0)
		

	def addPid(self,pid,lane):

		self.runpid[pid]=lane
			

# 並列処理each
def meach(func , para , *, mpCount=4 ,addcntNo=False,addprocNo=False ):

	params = copy.deepcopy(para)
	ttl    = len(params)
	addpara=0
	if addcntNo and addprocNo:
		addpara=3
	elif addcntNo :
		addpara=1
	elif addprocNo :
		addpara=2

	nowcnt = 0 

	mpm = MparallelManager(mpCount)

	while len(params)>0 :

		param=params.pop(0) 
		nowlane = mpm.getLane()
		pid=os.fork()
		if pid == 0:
			if addpara ==0 :
				func(param)
			elif addpara ==1 :
				func(param,nowcnt)
			elif addpara ==2 :
				func(param,nowlane)
			elif addpara ==3 :
				func(param,nowcnt,nowlane)
			os._exit(0)
		else:
			mpm.addPid(pid,nowlane)
		nowcnt+=1
		
	mpm.waitall()
	return []








#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil
import nysol.mcmd as nm
import nysol.util as nu

import nysol.util.mtemp as mtemp
import nysol.util.margs as margs


class mfriends(object):
	helpMSG="""
# ver="1.0" # 初期リリース 2016/10/2
# ver="1.1" -pal,rank2=,-udout追加 2016/12/13
# ver="1.2" rank2=削除,-palの意味を変更 2016/12/25
$cmd=$0.sub(/.*\//,"")
$version="1.2"
----------------------------
#{$cmd} version #{$version}
----------------------------
概要) 相互類似関係にある枝を選択する。
内容) グラフG=(V,E)の任意の枝(a,b)∈E について条件b∈A and a∈Bを満たす枝を選択する。
      ここで、A(B)は節点a(b)の隣接節点の内、類似度が上位r個の節点集合のこと。
      rはrank=で指定し、類似度(項目)はsim=で指定する。
      結果は有向グラフとして出力される。例えば節点a,bが相互類似関係にあれば、a->b,b->aの両枝が出力される。
書式) #{$cmd} ei= [ef=] [ni=] [nf=] eo= [no=] [sim=] [dir=b|m|x] [-directed] [T=] [--help]

  ファイル名指定
  ei=   : 枝データファイル
  ef=   : 枝データ上の2つの節点項目名(省略時は"node1,node2")
  ni=   : 節点データファイル
  nf=   : 節点データ上の節点項目名(省略時は"node")
  eo=   : データ研磨後の枝データファイル
  no=   : データ研磨後の節点データファイル
  rank= : 類似度上位何個までの隣接節点を対象とするか(省略時は3)
  sim=  : rank=で使う節点間類似度(枝の重み)項目名。
  dir=  : b:双方向類似枝のみ出力する(デフォルト)
        : m:片方向類似枝のみ出力する
        : x:双方向類似枝、片方向類似枝両方共出力する。
  -directed: 有向グラフとみなして計算する。
  -udout: 無向グラフとして出力する。両方向に枝がある場合(a->b,b->a)の枝はa-bとして出力される。
          a->b,b->aで類似度が異なる場合は平均値が出力される。

  その他
  T= : ワークディレクトリ(default:/tmp)
  --help : ヘルプの表示

例) 基本例
$ cat edge.csv
n1,n2,sim
a,b,0.40
a,c,0.31
a,d,0.31
b,c,0.20
b,d,0.24
b,e,0.14
c,d,0.30
d,e,0.09
$ #{$cmd} ei=edge.csv ef=n1,n2 sim=sim rank=2 eo=output.csv
#END# #{$cmd} ei=edge.csv ef=n1,n2 sim=sim rank=2 eo=output.csv; 2016/10/02 09:58:22
$ cat output.csv
n1%0,n2%1,sim
a,b,0.40
a,c,0.31
a,d,0.31
b,a,0.40
c,a,0.31
c,d,0.30
d,a,0.31
d,c,0.30

# Copyright(c) NYSOL 2012- All Rights Reserved.
		"""

	verInfo="version=1.2"
	paramter = {	
		"ni":"filename",
		"nf": "fld",  #<= 使ってない
		"ei":"filename",
		"ef": "fld",
		"eo":"filename",
		"no":"filename",
		"sim": "fld",
		"rank":"int",
		"dir":"str",
		"directed": "bool",
		"udout": "bool",
		"mcmdenv" : "bool",
		"T": "str"
	}

	paramcond = {	
		"hissu": ["ei","sim","eo"]
	}

	def help(self):
		print(mfriends.helpMSG)

	def ver(self):
		print(mfriends.versionInfo)


	def __param_check_set(self , kwd):

		for k,v in kwd.items():
			if not k in mfriends.paramter	:
				raise( Exception("KeyError: {} in {} ".format(k,self.__class__.__name__) ) )
			#型チェック入れる


		self.ei = kwd["ei"]
		self.ni = kwd["ni"]    if "ni"    in kwd else None
		
		# ---- edge field names (two nodes) on ei=
		ef0 = kwd["ef"].split(",")
		self.ef1 = ef0[0]
		self.ef2 = ef0[1] 
		
		# ---- node field name on ni=
		# self.nf  = kwd["nf"] if "nf" in kwd else None

		self.sim  = kwd["sim"] # similarity measure
		self.rank = int(kwd["rank"]) if "rank" in kwd else 3 # ranking 
		self.dir  = kwd["dir"] if "dir" in kwd else "b"  # 方向
		self.directed = kwd["directed"] if "directed" in kwd else False # directed graph
		self.udout    = kwd["udout"] if "udout" in kwd else False # 無向グラフ出力


		if(self.dir!="b" and self.dir!="m" and self.dir!="x"):
			raise Exception("dir= takes b, m, x")

		self.eo      = kwd["eo"]  if "eo"  in kwd else None
		self.no      = kwd["no"]  if "no"  in kwd else None 
		self.logFile = kwd["log"] if "log" in kwd else None 
	
	
	def __cmdline(self):
		cmdline = self.__class__.__name__
		for k,v in self.args.items():
			if type(v) is bool :
				if v == True :
					cmdline += " -" + str(k)
			else:
				cmdline += " " + str(k) + "=" + str(v)
		return cmdline 


	def __init__(self,**kwd):
		#パラメータチェック
		self.args = kwd
		self.__param_check_set(kwd)


	def run(self,**kw_args):

		os.environ['KG_ScpVerboseLevel'] = "2"
		if "msg" in kw_args:
			if kw_args["msg"] == "on":
				os.environ['KG_ScpVerboseLevel'] = "4"

		wf=mtemp.Mtemp()
		xxpal = wf.file()
		xxa   = wf.file()
		xxb   = wf.file()
		xxc   = wf.file()
		xxd   = wf.file()
		xxout = wf.file()

		# ============
		# n1,n2,sim
		# a,b,0.40
		# a,c,0.31
		# a,d,0.22
		# b,c,0.20
		# b,d,0.24
		# b,e,0.14
		# c,d,0.30
		# d,e,0.09
		xpal = None
		if self.directed :
			# 任意の枝a->bのaについて上位rankを選択
			xpal <<= nm.mnumber(k=self.ef1,s=self.sim+"%nr",e="skip",S=1,a="##rank",i=self.ei)
			xpal <<= nm.mselnum(f="##rank",c="[,"+str(self.rank)+"]")
		else:
			xxa = nm.mfsort(f=[self.ef1,self.ef2],i=self.ei)
			xxb = nm.mfsort(f=[self.ef2,self.ef1],i=self.ei)
			xpal <<= nm.muniq(k=[self.ef1,self.ef2],i=[xxa,xxb])
			xpal <<= nm.mnumber(k=self.ef1,s=self.sim+"%nr",e="skip",S=1,a="##rank")
			xpal <<= nm.mselnum(f="##rank",c="[,"+str(self.rank)+"]")


		# 両方向+片方向
		xpal1=None
		if self.dir=="x":
			xpal1 <<= nm.mcut(f=[self.ef1,self.ef2,self.sim],i=xpal)
		# 両方向
		elif self.dir=="b":
			selpara = "$s{%s}==$s{##ef2}"%(self.ef1)
			# 得られた上位rankグラフからa->b->cを作成し、a==cであれば相思相愛ということ
			xpal1 <<= nm.mnjoin(k=self.ef2,K=self.ef1,m=xpal,f=self.ef2+":##ef2,"+self.sim+":sim2",i=xpal)
			xpal1 <<= nm.msel(c=selpara)
			xpal1 <<= nm.mcut(f=[self.ef1,self.ef2,self.sim])
		else:
			selpara = "$s{%s}==$s{##ef2}"%(self.ef1)
			xxc = None
			xxc <<= nm.mnjoin(k=self.ef2,K=self.ef1,m=xpal,f=self.ef2+":##ef2,"+self.sim+":sim2",i=xpal)
			xxc <<= nm.msel(c=selpara)
			xxc <<= nm.mcut(f=[self.ef1,self.ef2])
			xpal1 <<= nm.mcut(f=[self.ef1,self.ef2,self.sim],i=xpal)
			xpal1 <<= nm.mcommon(k=self.ef1+","+self.ef2,m=xxc,r=True)

		runpal =None
		kpara="%s,%s"%(self.ef1,self.ef2)
		if self.udout :
			runpal <<= nm.mfsort(f=kpara,i=xpal1)
			runpal <<= nm.mavg(k=kpara,f=self.sim)
			runpal <<= nm.msortf(f=kpara,o=self.eo)
		else:
			runpal <<= nm.msortf(f=kpara,i=xpal1,o=self.eo)

		runpal.run()

		if self.ni and self.no :
			shutil.copyfile(self.ni,self.no)

		nu.mmsg.endLog(self.__cmdline())




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import os

import shutil
from nysol.util.mtemp import Mtemp
from nysol.util.mrecount import mrecount
import nysol.mcmd as nm

try:
	from igraph import *
except ImportError:
	sys.stderr.write("## warning: cannot import `igraph', so `Data.plot' method cannot be used for graph data")

try:
	import matplotlib.pyplot as plt
	import numpy as np
except ImportError:
	sys.stderr.write("## warning: cannot import `matplotlib' or `numpy', so `Data.plot' method cannot be used for transaction data ")

class Data(object):
	"""
	トランザクションデータ、およびグラフデータの様々なサンプルを生成する。

	**トランザクションデータ利用例:**

	.. doctest::

		>>> from take.sample import Data
		
		>>> dat=Data(name="tra00") # "tra00"という名前のトランザクションデータオブジェクトの生成
		>>> tadj=dat.getAdjMatrix() # 隣接行列形式(トランザクション毎のアイテムリスト)
		>>> print(tadj)
		[[2, 4], [3, 4, 5], [0, 1, 3, 5], [1, 3, 5], [0, 1, 3, 4], [0, 1, 3, 4, 5]]
		>>> tadj_a=dat.getAdjMatrix(alphabet=True) # ノードをアルファベットにした隣接行列形式(トランザクションIDをキーにした辞書型)
		>>> print(tadj_a)
		{'0': ['c', 'e'], '1': ['d', 'e', 'f'], '2': ['a', 'b', 'd', 'f'], '3': ['b', 'd', 'f'], '4': ['a', 'b', 'd', 'e'], '5': ['a', 'b', 'd', 'e', 'f']}
		>>> item=dat.getNodes() # item一覧を取得
		>>> print(item)
		[0, 1, 2, 3, 4, 5]
		>>> tra=dat.getEdges() # トランザクションIDとそれに含まれるアイテムの一覧を取得(以下の例では、トランザクションID=0に2と4のアイテムが含まれるということ)。
		>>> print(tra)
		[(0, 2), (0, 4), (1, 3), (1, 4), (1, 5), (2, 0), (2, 1), (2, 3), (2, 5), (3, 1), (3, 3), (3, 5), (4, 0), (4, 1), (4, 3), (4, 4), (5, 0), (5, 1), (5, 3), (5, 4), (5, 5)]
		>>> item_a=dat.getNodes(alphabet=True) # 上記と同様のことをアイテムをアルファベットにして出力(0から順に"a","b",...)
		>>> print(item_a)
		['a', 'b', 'c', 'd', 'e', 'f']
		>>> tra_a=dat.getEdges(alphabet=True) # トランザクションIDは整数の文字列、アイテムはアルファベット
		>>> print(tra_a)
		[('0', 'c'), ('0', 'e'), ('1', 'd'), ('1', 'e'), ('1', 'f'), ('2', 'a'), ('2', 'b'), ('2', 'd'), ('2', 'f'), ('3', 'b'), ('3', 'd'), ('3', 'f'), ('4', 'a'), ('4', 'b'), ('4', 'd'), ('4', 'e'), ('5', 'a'), ('5', 'b'), ('5', 'd'), ('5', 'e'), ('5', 'f')]
		>>> dat.writeAdjMatrix(oFile="smp_adj") # 隣接行列形式でファイル"smp_adj"に出力する
		>>> Data.head("smp_adj",top=0) # ファイルの上位top件を表示(top=0で全行表示)
		2 4
		3 4 5
		0 1 3 5
		1 3 5
		0 1 3 4
		0 1 3 4 5
		<BLANKLINE>
		>>> dat.writeAdjMatrix(oFile="smp_adja",alphabet=True)
		>>> Data.head("smp_adja",top=0)
		tid,item
		0,c e
		1,d e f
		2,a b d f
		3,b d f
		4,a b d e
		5,a b d e f
		<BLANKLINE>
		>>> dat.writeNodes(oFile="smp_items")
		>>> Data.head("items",top=0)
		0
		1
		2
		3
		4
		5
		<BLANKLINE>
		>>> dat.writeEdges(oFile="smp_tra")
		>>> Data.head("tra",top=0)
		0 2
		0 4
		1 3
		1 4
		1 5
		2 0
		2 1
		2 3
		2 5
		3 1
		3 3
		3 5
		4 0
		4 1
		4 3
		4 4
		5 0
		5 1
		5 3
		5 4
		5 5
		<BLANKLINE>
		>>> dat.writeNodes(oFile="smp_items_a",alphabet=True)
		>>> Data.head("items_a",top=0)
		node
		a
		b
		c
		d
		e
		f
		<BLANKLINE>
		>>> dat.writeEdges(oFile="smp_tra_a",alphabet=True)
		>>> Data.head("tra_a",top=0)
		tid,item
		0,c
		0,e
		1,d
		1,e
		1,f
		2,a
		2,b
		2,d
		2,f
		3,b
		3,d
		3,f
		4,a
		4,b
		4,d
		4,e
		5,a
		5,b
		5,d
		5,e
		5,f
		<BLANKLINE>
		>>> dat.plot(oFile="smp_tra00.png")

	.. figure:: ../../docs/samples/plot/tra00.png
		:scale: 100 %
		:alt: tra00.png

		tra00.png

		**グラフデータ利用例:**

	.. doctest::

		>>> dat=Data(name="graph00") # "tra00"という名前のトランザクションデータオブジェクトの生成
		>>> adj=dat.getAdjMatrix() # 隣接行列
		>>> print(adj)
		[[1, 4, 2], [2, 4, 6], [3, 5], [4], [5], [], []]
		>>> adj_a=dat.getAdjMatrix(alphabet=True) # ノードをアルファベットにした隣接行列
		>>> print(adj_a)
		{'a': ['b', 'e', 'c'], 'b': ['c', 'e', 'g'], 'c': ['d', 'f'], 'd': ['e'], 'e': ['f'], 'f': [], 'g': []}
		>>> nodes=dat.getNodes() # ノード一覧を取得
		>>> print(nodes)
		[0, 1, 2, 3, 4, 5, 6]
		>>> edges=dat.getEdges() # エッジ一覧(ノードペア)を取得
		>>> print(edges)
		[(0, 1), (0, 4), (0, 2), (1, 2), (1, 4), (1, 6), (2, 3), (2, 5), (3, 4), (4, 5)]
		>>> nodes_a=dat.getNodes(alphabet=True) # 上記と同様のことをアイテムをアルファベットにして出力(0から順に"a","b",...)
		>>> print(nodes_a)
		['a', 'b', 'c', 'd', 'e', 'f', 'g']
		>>> edges_a=dat.getEdges(alphabet=True)
		>>> print(edges_a)
		[('a', 'b'), ('a', 'e'), ('a', 'c'), ('b', 'c'), ('b', 'e'), ('b', 'g'), ('c', 'd'), ('c', 'f'), ('d', 'e'), ('e', 'f')]
		>>> dat.writeAdjMatrix(oFile="smp_adj") # 隣接行列形式でファイル"adj"に出力する
		>>> Data.head("adj",top=0) # ファイルの上位top件を表示(top=0で全行表示)
		1 4 2
		2 4 6
		3 5
		4
		5
		<BLANKLINE>
		<BLANKLINE>
		<BLANKLINE>
		>>> dat.writeAdjMatrix(oFile="smp_adj_a",alphabet=True)
		>>> Data.head("adj_a",top=0)
		node,adjacency
		a,b e c
		b,c e g
		c,d f
		d,e
		e,f
		f,
		g,
		<BLANKLINE>
		>>> dat.writeNodes(oFile="smp_nodes")
		>>> Data.head("nodes",top=0)
		0
		1
		2
		3
		4
		5
		6
		<BLANKLINE>
		>>> dat.writeNodes(oFile="smp_nodes_a",alphabet=True)
		>>> Data.head("nodes_a",top=0)
		node
		a
		b
		c
		d
		e
		f
		g
		<BLANKLINE>
		>>> dat.writeEdges(oFile="smp_edges")
		>>> Data.head("edges",top=0)
		0 1
		0 4
		0 2
		1 2
		1 4
		1 6
		2 3
		2 5
		3 4
		4 5
		<BLANKLINE>
		>>> dat.writeEdges(oFile="smp_edges_a",alphabet=True)
		>>> Data.head("edges_a",top=0)
		node1,node2
		a,b
		a,e
		a,c
		b,c
		b,e
		b,g
		c,d
		c,f
		d,e
		e,f
		<BLANKLINE>
		>>> dat.plot(oFile="smp_graph00.png")


	.. figure:: ../../docs/samples/plot/graph00.png
		:scale: 100 %
		:alt: graph00.png

		"graph00.png"

	"""

	def  __init__(self,name):
		self.name=name
		eval("self._{}()".format(self.name))
		self._parseData()

	def _parseData(self):
		if self.type=="tra":
			self.nodes=[]
			self.edges=[]
			for t in range(len(self.data)):
				for item in self.data[t]:
					self.nodes.append(int(item))
					self.edges.append((t,int(item)))
			self.nodes=list(set(self.nodes))
		else:
			self.nodes=[]
			self.edges=[]
			for n in range(len(self.data)):
				self.nodes.append(n)
				for adj in self.data[n]:
					self.edges.append((n,int(adj)))

	def _num2alpha(val):
		s=""
		while(True):
			div=int(val/26)
			sub=val%26
			s+=chr(97+sub)
			if(div<=0):
				break
			val=div-1
		return s[::-1] # reverse order

	def getAdjMatrix(self,alphabet=False):
		if alphabet:
			adjMatrix={}
			for n in range(len(self.data)):
				if self.type=="tra":
					ns=str(n)
				else:
					ns=Data._num2alpha(n)
				adjMatrix[ns]=[]
				for adj in self.data[n]:
					adjMatrix[ns].append(Data._num2alpha(adj))
			return adjMatrix

		else:
			return self.data

	def getNodes(self,alphabet=False):
		if alphabet:
			n=[]
			for node in self.nodes:
				n.append(Data._num2alpha(node))
			return n
		return self.nodes

	def getEdges(self,alphabet=False):
		if alphabet:
			e=[]
			for edge in self.edges:
				if self.type=="tra":
					e.append((str(edge[0]),Data._num2alpha(edge[1])))
				else:
					e.append((Data._num2alpha(edge[0]),Data._num2alpha(edge[1])))
			return e
		return self.edges

	def writeAdjMatrix(self,oFile,alphabet=False):
		with open(oFile,"w") as fpw:
			if alphabet:
				if self.type=="tra":
					fpw.write("tid,item\n")
				else:
					fpw.write("node,adjacency\n")

			for n in range(len(self.data)):
				if alphabet:
					if self.type=="tra":
						ns=str(n)
					else:
						ns=Data._num2alpha(n)
					fpw.write(ns+",")

				ele=[]
				for adj in self.data[n]:
					if alphabet:
						ele.append(Data._num2alpha(adj))
					else:
						ele.append(str(adj))
				fpw.write(" ".join(ele))
				fpw.write("\n")

	def writeNodes(self,oFile,alphabet=False):
		nodes=self.getNodes(alphabet=alphabet)
		with open(oFile,"w") as fpw:
			if alphabet:
				fpw.write("node\n")
			for node in nodes:
				fpw.write(str(node))
				fpw.write("\n")

	def writeEdges(self,oFile,alphabet=False):
		"""
		CSVファイルに出力する。
		form="edge"でノードペアのエッジを一行として出力し、
		form="adjacent"で隣接行列形式で出力する。

		.. code-block:: python
			:linenos:

			>>> dat=Data("numGraph1")
			>>> n,e=dat.getList()
			>>> print(n)
			['0', '1', '2', '3', '4', '5', '6']
			>>> print(e)
			[(0, 1), (0, 2), (0, 4), (1, 2), (1, 4), (1, 6), (2, 3), (2, 6), (3, 4), (4, 5)]
			>>> dat.plot()
			>>> dat.write("edge"," ","g.txt")
			>>> Data.head("g.txt")

		"""

		delim="," if alphabet else " "
		edges=self.getEdges(alphabet=alphabet)
		with open(oFile,"w") as fpw:
			if alphabet:
				if self.type=="tra":
					fpw.write("tid,item\n")
				else:
					fpw.write("node1,node2\n")
			for edge in edges:
				fpw.write(str(edge[0])+delim+str(edge[1]))
				fpw.write("\n")

	def plot(self,alphabet=False,oFile=None):
		if self.type=="tra":
			row_labels=[]
			col_labels=[]
			for tra in range(len(self.data)):
				row_labels.append(str(tra))
			for item in range(len(self.nodes)):
				if alphabet:
					col_labels.append(Data._num2alpha(item))
				else:
					col_labels.append(str(item))

			row_size=len(self.data)
			col_size=len(self.nodes)
			mtx=np.full((row_size,col_size),0)
			for tra in range(len(self.data)):
				for item in self.data[tra]:
					mtx[tra][item]=1

			fig, ax = plt.subplots(figsize=(row_size/3,col_size/3))
			heatmap = ax.pcolor(mtx, cmap=plt.cm.Blues)

			ax.set_xticks(np.arange(mtx.shape[0]) + 0.5, minor=False)
			ax.set_yticks(np.arange(mtx.shape[1]) + 0.5, minor=False)
			ax.invert_yaxis()
			ax.xaxis.tick_top()
			ax.set_xticklabels(col_labels, minor=False)
			ax.set_yticklabels(row_labels, minor=False)
			if oFile==None:
				plt.show()
			else:
				plt.savefig(oFile)

		else:
			g = Graph(edges=self.edges)
			vStyle = {}
			vStyle["vertex_size"] = 25
			vStyle["vertex_color"] = ["white" for gender in self.nodes]
			if alphabet:
				n=[]
				for node in self.nodes:
					n.append(Data._num2alpha(node))
				vStyle["vertex_label"] = n
			else:
				vStyle["vertex_label"] = self.nodes

			vStyle["edge_width"] = [1 for i in self.edges]
			vStyle["layout"] = g.layout("kamada_kawai")
			vStyle["bbox"] = (200, 200)
			vStyle["margin"] = 30
			if oFile==None:
				plot(g,**vStyle)
			else:
				plot(g,oFile,**vStyle)

	def head(fName,top=6):
		try:
			with open(fName) as fpr:
				c=0
				for line in fpr:
					if top==0 or c<top:
						print(line.strip())
						c+=1
					else:
						break
				print("")
		except (RuntimeError, TypeError, NameError):
			pass

	def _graph00(self):
		self.type="graph"
		self.data=[
		  [1,4,2],
		  [2,4,6],
		  [3,5],
		  [4],
		  [5],
		  [],
		  []
		]

	def _graph01(self):
		self.type="graph"
		self.data=[
		  [1,4,2],
		  [4,2],
		  [3],
		  [],
		  [5,3],
		  [2],
		  []
		]

	def _tra00(self):
		self.type="tra"
		self.data=[
			[2,4],
			[3,4,5],
			[0,1,3,5],
			[1,3,5],
			[0,1,3,4],
			[0,1,3,4,5]
		]

	def _numTraDB1w(self,fName="numTraDB1w.csv"):
		oFile="{0}/{1}".format(self.oPath,fName)
		data='''-1
1
-1
1
1
-1
'''
		self._writeFile(data,oFile)
		return oFile


if __name__ == '__main__':

	"""
	dat=Data("tra00")

	tadj=dat.getAdjMatrix()
	tadja=dat.getAdjMatrix(alphabet=True)
	print(tadj)
	print(tadja)
	item=dat.getNodes()
	tras=dat.getEdges()
	print(item)
	print(tras)
	item_a=dat.getNodes(alphabet=True)
	tras_a=dat.getEdges(alphabet=True)
	print(item_a)
	print(tras_a)

	dat.writeAdjMatrix(oFile="xxtadj")
	dat.writeAdjMatrix(oFile="xxtadja",alphabet=True)
	dat.writeNodes(oFile="xxitems")
	dat.writeNodes(oFile="xxitemsa",alphabet=True)
	dat.writeEdges(oFile="xxtras")
	dat.writeEdges(oFile="xxtrasa",alphabet=True)
	Data.head("xxtadj",top=0)
	Data.head("xxtadja",top=0)
	Data.head("xxitems",top=0)
	Data.head("xxitemsa",top=0)
	Data.head("xxtras",top=0)
	Data.head("xxtrasa",top=0)

	dat.plot(oFile="xxtra.png")
	dat.plot(oFile="xxtraa.png",alphabet=True)
	"""


	dat=Data("graph00")

	adj=dat.getAdjMatrix()
	print(adj)
	adja=dat.getAdjMatrix(alphabet=True)
	print(adja)

	node=dat.getNodes()
	edge=dat.getEdges()
	print(node)
	print(edge)
	nodea=dat.getNodes(alphabet=True)
	edgea=dat.getEdges(alphabet=True)
	print(nodea)
	print(edgea)

	dat.writeAdjMatrix(oFile="xxadj")
	dat.writeAdjMatrix(oFile="xxadja",alphabet=True)
	dat.writeNodes(oFile="xxnodes")
	dat.writeNodes(oFile="xxnodesa",alphabet=True)
	dat.writeEdges(oFile="xxedges")
	dat.writeEdges(oFile="xxedgesa",alphabet=True)
	Data.head("xxadj",top=0)
	Data.head("xxadja",top=0)
	Data.head("xxnodes",top=0)
	Data.head("xxnodesa",top=0)
	Data.head("xxedges",top=0)
	Data.head("xxedgesa",top=0)

	dat.plot(oFile="xxg1.png")
	dat.plot(oFile="xxg1a.png",alphabet=True)





# -*- coding: utf-8 -*-
import shlex
import re
import copy
import os, sys

import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib import nysolutil as nutil
from nysol.mcmd.nysollib import draw as ndraw
from nysol.mcmd.nysollib import dspalign
from nysol.mcmd.nysollib import itermod


class NysolMOD_CORE(object):

	OutMsg   = False
	RunLimit = -1
		
	def __init__(self,name=None,kwd=None) :

		self.name     = name
		self.addBySys = False
		self.msg      = False
		self.runlimit = -1
		self.kwd      = {}
		self.inplist  = {}
		self.outlist  = {}
		self.stdodir  = None
		self.stdidir  = None

		self.disabled_ouputlist = False
		self.modWeight = 0

		
		if hasattr(self,'_inkwd') and len(self._inkwd)>0 :
			self.modWeight += len(self._inkwd) * 4
			self.stdidir = self._inkwd[0].rstrip("=")
			for k in self._inkwd:
				self.inplist[k.rstrip("=")] = []

		if hasattr(self,'_outkwd')  and len(self._outkwd)>0 :

			self.modWeight += len(self._outkwd) * 1

			self.stdodir = self._outkwd[0].rstrip("=")
			for k in self._outkwd:
				self.outlist[k.rstrip("=")] = []
				
		else:

			self.disabled_ouputlist = True
		
		if hasattr(self,'_disabled_ouputlist'):
			self.disabled_ouputlist = self._disabled_ouputlist

		if self.modWeight == 0:
			self.modWeight += 1

		if kwd != None:
			self.kwd   = kwd

		self.tag = ""
		self.nowdir   = self.stdodir
		self.nowidir  = self.stdidir

		if "tag" in self.kwd :

			self.tag = self.kwd["tag"]
			del self.kwd["tag"]

		if "sysadd" in self.kwd :

			self.addBySys = self.kwd["sysadd"]
			del self.kwd["sysadd"]

		for key in self.inplist.keys():

			if key in self.kwd :
				
				if isinstance(self.kwd[key],list):

					complexFlg = True

					if isinstance(self.kwd[key][0],list):	

						if isinstance(self.kwd[key][0][0],(str,int,float)):

							complexFlg = False
						
					if complexFlg:

						for kval in self.kwd[key]:

							if isinstance(kval,NysolMOD_CORE):	
								kval.outlist[kval.nowdir].append(self)

							self.inplist[key].append(kval)
					
					else:

						if isinstance(self.kwd[key],NysolMOD_CORE):	
							self.kwd[key].outlist[self.kwd[key].nowdir].append(self)

						self.inplist[key].append(self.kwd[key])


				elif isinstance(self.kwd[key],NysolMOD_CORE):	

					self.kwd[key].outlist[self.kwd[key].nowdir].append(self)
					self.inplist[key].append(self.kwd[key])

				else:

					self.inplist[key].append(self.kwd[key])
		

				del self.kwd[key]
		

		for key in self.outlist.keys():
	
			if key in self.kwd :

				self.outlist[key].append(self.kwd[key])
				del self.kwd[key]


	def redirectsimple(self,dir) :
		self.nowdir = dir
		return self

	def iredirect(self,dir) :
		self.nowidir = dir
		return self


	def redirect(self,dir) :
		olddir = self.nowdir 
		self.nowdir = dir
		from nysol.mcmd.submod.mfifo import Nysol_Mfifo as mfifo
		fifoxxx=mfifo(i=self,sysadd=True)
		self.nowdir = olddir
		return fifoxxx

	def okwdObjCnt(self):
		count=0
		for key in self.outlist.keys():
			count += len(self.outlist[key])	
		return count


	def _dspselct(self,ylim,xlim=20):

		yLimit = ylim
		pre=[]
		head=None
		sufmax = int(yLimit/2)
		xmid = int(xlim/2)

		suf=[ [] for i in range(sufmax) ]
		cnt=0
		sufpos=0

		dupobj = copy.deepcopy(self)

		# 不要 mod 除去 
		itobj = dupobj.redirect(dupobj.nowdir)
		#for k in dupobj.outlist.keys():
		#	dupobj.outlist[k].clear()


		try:
			xx = itermod.LineListIter(itobj)
			if len(xx.fldname)>xlim:
				head = xx.fldname[:xmid]+["..."] + xx.fldname[-xmid:]
			else:
				head = xx.fldname 

			while(True):
				val = next(xx)
				if val==None:
					break
				xval = val
				if len(val)>xlim:
					xval = val[:xmid]+["..."] + val[-xmid:]

				if cnt < yLimit :
					pre.append(xval)

				cnt+=1
				suf[sufpos]=xval
				sufpos+=1
				if sufpos==sufmax :
					sufpos=0

		except StopIteration:
			pass

		if(cnt > yLimit): 
			ppos = sufmax
			spos = sufpos
			for _ in range(sufmax):

				pre[ppos] = suf[spos]
				ppos += 1
				spos += 1
				if spos==sufmax :
					spos=0

		return pre,cnt,head


	def _dsp1(self):

		yLimit =40

		dspdata ,cnt ,head = self._dspselct(yLimit)

		outstrList = dspalign.chgDSPstr(dspdata , cnt > yLimit)

		return "\n".join(outstrList)



	def _dsp2(self):

		yLimit =40

		dspdata ,cnt ,head = self._dspselct(yLimit)

		outstrList = dspalign.chgDSPhtml(dspdata , yLimit , cnt, head)

		return "\n".join(outstrList)



	def __str__(self):
		import os

		dsptp = os.getenv("NYSOL_MOD_DSP_TYPE", "0")


		if dsptp == "1":
		
			return self._dsp1()

		else:

			return "<{} at {}>".format(self.__class__.__name__,hex(id(self)))


	def __repr__(self):

		return str(self)

	def _repr_html_(self):
		return self._dsp2()

	def __iter__(self):

		try:
			x = itermod.LineListIter(self)
			while(True):
				v = next(x)
				if v==None:
					break
				yield v

			return 

		except GeneratorExit:
			n_core.close(x.csvin)
			n_core.cancel(x.shobj)

	def fldname(self):
		dupobj = copy.deepcopy(self)

		# 不要 mod 除去 
		itobj = dupobj.redirect(dupobj.nowdir)


		xx = itermod.LineListIter(itobj)
		head = xx.fldname
		
		n_core.close(xx.csvin)
		n_core.cancel(xx.shobj)
		return head
		

	# __getLineList処理は同じ
	def convtype(self,dtype=None,header=False):
		try:
			x = itermod.LineListIter(self,dtype)
			while(True):
				v = next(x)
				if v==None:
					break
				yield v
			return 

		except GeneratorExit:
			n_core.close(x.csvin)
			n_core.cancel(x.shobj)

	def __getLineList(self,dtype=None,skey=None,q=False,header=False):

		try:
			x = itermod.LineListIter(self,dtype,skey,q,header)
			while(True):
				v = next(x)
				if v==None:
					break
				yield v
			return 

		except GeneratorExit:
			n_core.close(x.csvin)
			n_core.cancel(x.shobj)

	def __getLineListWithInfo(self,keys,skeys=None,dtype=None,q=False,header=False):
		try:
			x = itermod.LineListIterWithInfo(self,keys,skeys,dtype,q,header)

			while(True):
				v = next(x)
				if v==None:
					break
				yield v
			return 

		except GeneratorExit:
			n_core.close(x.csvin)
			n_core.cancel(x.shobj)


	def __getLineDictWithInfo(self,keys,skeys=None,dtype=None,q=False):

		try:
			x = itermod.LineDictIterWithInfo(self,keys,skeys,dtype,q)
			while(True):
				v = next(x)
				if v==None:
					break
				yield v
			return 

		except GeneratorExit:
			n_core.close(x.csvin)
			n_core.cancel(x.shobj)

	def __getLineDict(self,dtype=None,skey=None,q=False):

		try:

			x = itermod.LineDictIter(self,dtype,skey,q)
			while(True):
				v = next(x)
				if v==None:
					break
				yield v
			return 

		except GeneratorExit:
			n_core.close(x.csvin)
			n_core.cancel(x.shobj)




	# return generator
	# def getline(self,dtype=None,keys=None,skeys=None,otype="list",q=False,header=False):
	def getline(self,dtype=None,k=None,s=None,otype="list",q=False,header=False):

		if otype == "list":

			if k!=None:

				return self.__getLineListWithInfo(k,s,dtype,q,header)

			elif s!=None:

				return self.__getLineList(dtype,s,q,header)

			else:
				return self.__getLineList(dtype,None,q,header)


		elif otype == "dict":

			if k!=None :

				return self.__getLineDictWithInfo(k,s,dtype,q)

			elif s!=None:

				return self.__getLineDict(dtype,s,q)

			else:
				
				return self.__getLineDict(dtype,None)

		else:
			wxc = Exception("unsuport rtype" + rtype)

			raise wxc




	## generator rap
	def __getBlockList(self,keys,skeys=None,dtype=None,q=False,header=False):

		try:

			x = itermod.BlkListIter(self,keys,skeys,dtype,q,header)
			while(True):
				v = next(x)
				if v==None:
					break
				yield v
			return 

		except GeneratorExit:

			n_core.close(x.csvin)
			n_core.cancel(x.shobj)

	def __getBlockDict(self,keys,skeys=None,dtype=None,q=False):

		try:

			x = itermod.BlkDictIter(self,keys,skeys,dtype,q)
			while(True):
				v = next(x)
				if v==None:
					break
				yield v
			return 

		except GeneratorExit:

			n_core.close(x.csvin)
			n_core.cancel(x.shobj)

	# return generator
	def keyblock(self,k,s=None,dtype=None,otype="list",q=False,header=False):

		if otype == "list":

			return self.__getBlockList(k,s,dtype,q,header)

		elif otype == "dict":
			
			return self.__getBlockDict(k,s,dtype,q)

		else : 

			raise Exception("unsuport rtype" + otype)


	""" F.w. 
	def set_runlimit(self,lim):
		self.runlimit=int(lim)
		return self

	def msgOn(self):
		self.msg=True
		return self

	def msgOff(self):
		self.msg=False
		return self
	"""
	

	def addPre(self,pre): 

		self.inplist[self.nowidir].append(pre) 
		pre.outlist[pre.nowdir].append(self)
		return self

	def __ilshift__(self, other):

		pre = other
		precnt = len(pre.inplist[pre.nowidir])

		while precnt != 0:

			ck_o_pos = -1
			ck_o_cnt = 0

			for i,ck_o in enumerate(pre.inplist[pre.nowidir]):
				if isinstance(ck_o,NysolMOD_CORE):
					ck_o_pos  = i
					ck_o_cnt += 1
					
			if ck_o_cnt > 1 :
				raise Exception("unsuport Multiple nysol flow object input")
			
			if ck_o_cnt == 0 :
				break

			pre = pre.inplist[pre.nowidir][ck_o_pos]
			precnt = len(pre.inplist[pre.nowidir])

		pre.addPre(self)
		return other

	def __rlshift__(self, other):

		if(other!=None):
			raise Exception("unsuport TYPE")

		return self

	def paraUpdate(self,kw):
		# f.w キーワードチェック入れる

		self.kwd.update(kw)

		for key in self.inplist.keys():

			if key in self.kwd :

				self.inplist[key].append(self.kwd[key])
				del self.kwd[key]


		for key in self.outlist.keys():

			if key in self.kwd :

				self.outlist[key].append(self.kwd[key])
				del self.kwd[key]

	@classmethod
	def setMsgFlg(self,val):
		NysolMOD_CORE.OutMsg = val

	@classmethod
	def setRunLimit(self,val):
		NysolMOD_CORE.RunLimit = int(val)


	@classmethod
	def check_dupObjSub(self,sumiobj,dupobj,obj):

		if obj in sumiobj:
			if obj in dupobj:
				dupobj[obj] += 1
			else:
				dupobj[obj] = 2
			return True 
		else:
			if not isinstance(obj,str):
				sumiobj.add(obj)
			return False
	
	def check_dupObj(self,sumiobj,dupobj):
		#o=でチェックできるはずなので　いらなくなるはず 		
		if NysolMOD_CORE.check_dupObjSub(sumiobj,dupobj,self) ==True :
			return

		for key in self.inplist.keys():

			if len(self.inplist[key]) != 0 :

				for xval in self.inplist[key]:

					if isinstance(xval,NysolMOD_CORE):
						xval.check_dupObj(sumiobj,dupobj)

					elif isinstance(xval,str):
						self.check_dupObjSub(sumiobj,dupobj,xval)

					elif isinstance(xval,list):
						pass

		return			
 
	
	def __addFifo(self,k):

		if len(self.outlist[k]) != 1:
			raise Exception("unsuporot size")

		outll = self.outlist[k][0]

		if outll.name == "mfifo":
			return

		kwd= None
		pos= None
		
		for ki in outll.inplist: # 0だけOK?
			if len(outll.inplist[ki])!=0 :						
				for ii in range(len(outll.inplist[ki])):
					if self == outll.inplist[ki][ii]:
						kwd = ki
						pos = ii
						
		if kwd == None or kwd == None:
			raise Exception("no vaild network")

		self.outlist[k] = []
		fifoxxx = self.redirect(k)
		fifoxxx.outlist[fifoxxx.nowdir]=[outll]
		outll.inplist[kwd][pos] = fifoxxx

	def __addTee(self,k):

		opos = []
		kwd  = []
		pos  = []
		sumiobj=set([])

		outll = self.outlist[k]

		for i in range(len(outll)):
			if outll[i] in sumiobj:
				continue
			sumiobj.add(outll[i])
			
			for ki in outll[i].inplist:

				for ii in range(len(outll[i].inplist[ki])):
					if self == outll[i].inplist[ki][ii]:
						opos.append(i)
						kwd.append(ki)
						pos.append(ii)

		from nysol.mcmd.submod.m2tee import Nysol_M2tee as m2tee
		
		ttl = len(opos)

		stktee = []
		for p in range(len(opos)):

			if p % 32 ==0 :
				teexxx = m2tee(sysadd=True)
				teexxx.outlist[teexxx.nowdir] = [] 
				stktee.append(teexxx)

			i = opos[p]
			ki = kwd[p]
			ii = pos[p]

			fifoxxx = teexxx.redirect(teexxx.nowdir)
			fifoxxx.outlist[fifoxxx.nowdir]=[ outll[opos[i]] ]   
			outll[opos[i]].inplist[ki][ii] = fifoxxx

		
		while len(stktee) != 1:
			orgtee = stktee
			stktee = []
			stpos = 0
			edpos = len(orgtee) if len(orgtee) < 32 else 32

			while stpos < len(orgtee):
				teexxx = m2tee(sysadd=True)
				teexxx.outlist[teexxx.nowdir] = orgtee[stpos:edpos]
				for xpos in range(stpos,edpos):
					orgtee[xpos].inplist["i"] = [teexxx]
					
				stktee.append(teexxx)
				stpos  = edpos
				edpos = len(orgtee) if len(orgtee) < stpos + 32 else stpos + 32

		self.outlist[k] = []
		save_nowdir = self.nowdir
		self.nowdir = k
		stktee[0].inplist["i"] = [self]
		self.outlist[k] = [ stktee[0] ]
		self.nowdir = save_nowdir
		
 
	@classmethod
	def addTee(self,dupobj): 

		addobj =[]

		for obj in dupobj:
			for k in obj.outlist.keys():

				if len(obj.outlist[k])==0:
					continue
				elif len(obj.outlist[k])==1: #fifoのみ追加(o,u対策)

					if isinstance(obj.outlist[k][0],str):
						continue 

					if isinstance(obj.outlist[k][0],list): #多分いらん

						from nysol.mcmd.submod.writelist import Nysol_Writelist as mwritelist
						if "nfn" in obj.kwd and obj.kwd["nfn"] == True :
							wobj = mwritelist(obj.outlist[k][0],sysadd=True,nfn=True)
						else:
							wobj = mwritelist(obj.outlist[k][0],sysadd=True)

						wobj.inplist[wobj.nowidir]=[obj]
						obj.outlist[k][0] = wobj
						addobj.append(wobj)
						continue 

					obj.__addFifo(k)

				else:

					obj.__addTee(k)

		return addobj


		# no buffer Version
		#for obj in dupobj:
		#	outll = obj.outlist["o"]
		#	teexxx = m2tee(i=obj)
		#	obj.outlist["o"]= [teexxx]
		#	teexxx.outlist["o"] = outll
		#	for outin in outll:
		#		if len(outin.inplist["i"])!=0 and obj == outin.inplist["i"][0]:
		#			outin.inplist["i"] = [teexxx]
		#		if len(outin.inplist["m"])!=0 and obj == outin.inplist["m"][0]:
		#			outin.inplist["m"] = [teexxx]




	@classmethod
	def addIoMod(self,sumiobj,dupobj):
		# add list read
		from nysol.mcmd.submod.readlist import Nysol_Readlist as mreadlist
		from nysol.mcmd.submod.writelist import Nysol_Writelist as mwritelist
		from nysol.mcmd.submod.m2cat import Nysol_M2cat as m2cat
		add_mod =[]

		for obj in sumiobj:

			if isinstance(obj,NysolMOD_CORE):
				if obj.name=="readlist":
					continue
				if obj.name=="writelist":
					continue

				for key in obj.inplist.keys():

					for i,xval in enumerate(obj.inplist[key]):

						if isinstance(xval,list) :

							rlmod = mreadlist(xval,sysadd=True,nfn=True)
							rlmod.outlist[rlmod.nowdir] = [obj]
							obj.inplist[key][i]=rlmod

					if len(obj.inplist[key])>1:

						ttl = len(obj.inplist[key])
						
						layer=1
						while 32 < ttl:
							if ttl%32 :
								ttl = int(ttl/32)
							else:
								ttl = int(ttl/32)+1
							layer += 1 
						
						orginplist = obj.inplist[key]
						
						for i in reversed(range(layer)):

							if i == 0 :
								if obj.name=="m2cat":
									obj.inplist["i"] = orginplist
								
								else:
									m2cmod  = m2cat(sysadd=True)
									m2cmod.inplist["i"] = orginplist
									m2cmod.outlist[m2cmod.nowdir] = [obj]

									for xval in orginplist:
										#if not isinstance(xval,NysolMOD_CORE):	
										#	raise Exception("can not mutli input except nysol moudule")

										if isinstance(xval,NysolMOD_CORE):
											#	raise Exception("can not mutli input except nysol moudule")
							
											for okey in xval.outlist.keys():
							
												for ii in range(len(xval.outlist[okey])):
													if obj == xval.outlist[okey][ii]:
														xval.outlist[okey][ii] = m2cmod
														break

									obj.inplist[key] = [m2cmod]

							else:
								lttl = len(orginplist)
								rem = lttl%32
								div = int(lttl/32)
								if rem != 0:
									div +=1	
									setend = rem 
								else:
									setend = 32 

								newinplist = [None] * div
								setbgn = 0
								
								for midi in range(div):


									orginpsuv = [None] * (setend-setbgn)
									for midj in range(setend-setbgn):
										orginpsuv[midj] = orginplist[setbgn+midj]

									m2cmod  = m2cat(sysadd=True)
									m2cmod.inplist["i"] = orginpsuv
									m2cmod.outlist[m2cmod.nowdir] = [obj]


									for xval in orginpsuv:
										#if not isinstance(xval,NysolMOD_CORE):	
										#	raise Exception("can not mutli input except nysol moudule")
										if isinstance(xval,NysolMOD_CORE):
							
											for okey in xval.outlist.keys():
								
												for ii in range(len(xval.outlist[okey])):
													if obj == xval.outlist[okey][ii]:
														xval.outlist[okey][ii] = m2cmod
														break

									newinplist[midi] = m2cmod
									setbgn  = setend
									setend += 32

								orginplist = newinplist


				for key in obj.outlist.keys():

					# 途中OUTPUTチェック
					if len(obj.outlist[key]) > 1:

						for i in range(len(obj.outlist[key])):

							if isinstance(obj.outlist[key][i],str) :

								from nysol.mcmd.submod.writecsv import Nysol_Writecsv as mwritecsv
								wcsv_o = mwritecsv(obj.outlist[key][i],sysadd=True)
								wcsv_o.inplist[wcsv_o.nowidir]=[obj]
								obj.outlist[key][i] = wcsv_o
								if obj in dupobj:
									dupobj[obj] += 1
								else:
									dupobj[obj] = 2
								add_mod.append(wcsv_o)

							if isinstance(obj.outlist[key][i],list) :
								if "nfn" in obj.kwd and obj.kwd["nfn"] == True :
									wlmod = mwritelist(obj.outlist[key][i],sysadd=True ,nfn=True)
								else:
									wlmod = mwritelist(obj.outlist[key][i],sysadd=True)
								
								wlmod.inplist[wlmod.nowidir]=[obj]

								obj.outlist[key][i] = wlmod
								if obj in dupobj:
									dupobj[obj] += 1
								else:
									dupobj[obj] = 2
								add_mod.append(wlmod)	

		return add_mod		

	
	@classmethod
	def change_modNetworks(self,mods):

		sumiobj=set([])
		dupobj={}
		for mod in mods:
			mod.check_dupObj(sumiobj,dupobj)

		add_Dmod = NysolMOD_CORE.addIoMod(sumiobj,dupobj)

		if len(dupobj)!=0:
			add_DmodTee = NysolMOD_CORE.addTee(dupobj)
			mods.extend(add_DmodTee)
		
		mods.extend(add_Dmod)


	def change_modNetwork(self):

		NysolMOD_CORE.change_modNetworks([self])

	def selectUniqMod(self,sumiobj,modlist):
		if self in sumiobj:
			return None

		pos = len(sumiobj)
		sumiobj.add(self)
		modlist[self]=pos
		
		for k in self.inplist.keys():
			for obj in self.inplist[k]:
				if isinstance(obj,NysolMOD_CORE):
					obj.selectUniqMod(sumiobj,modlist)



	def checkRmObj(self,kwd , pos ,mods):

		if len(mods) == 1:
			return True
		
		mod = self.outlist[kwd][pos]

		for ckmod in mods:
			if ckmod == mod :
				return False

		for k in mod.outlist.keys():
			for i,e in  enumerate(mod.outlist[k]):
				if isinstance(e,NysolMOD_CORE):
					  
					if not mod.checkRmObj(k,i,mods):
						return False
					
		return True



	@classmethod
	def makeModList(self,uniqmod,modlist,iolist): # iolist標準化

		for obj,no in uniqmod.items():
			# name,para,ipara,opara,tag
			modlist[no]= [obj.name,nutil.para2str(obj.kwd),{},{},obj.tag,obj.addBySys,obj.modWeight]
			iolist[no]=[[],[]]

			for k in obj.inplist.keys():
				datacheck = 0
				strstk=[]
				for ioobj in obj.inplist[k]:
					#uniqmodに無ければ今回のルート外のはず
					if isinstance(ioobj,NysolMOD_CORE) and ioobj in uniqmod :
						iolist[no][0].append([uniqmod[ioobj],k])
					elif isinstance(ioobj,str):
						if datacheck > 1:
							raise Exception("unsuppot mix input (str and list)")
						strstk.append(ioobj)
						datachek=1
					elif isinstance(ioobj,list):
						if datacheck != 0:
							raise Exception("unsuppot mix input (str and list) or multi list ")
						modlist[no][2][k]=ioobj
						datachek=2
				if len(strstk) != 0:
					modlist[no][2][k]=",".join(strstk)

			for k in obj.outlist.keys():

				for ioobj in obj.outlist[k]:
					#uniqmodに無ければ今回のルート外のはず
					if isinstance(ioobj,NysolMOD_CORE) and ioobj in uniqmod :
						iolist[no][1].append([uniqmod[ioobj],k])
					elif isinstance(ioobj,(list,str)):
						modlist[no][3][k]=ioobj
						

	@classmethod
	def getLink(self,iolist,base,to):

		for v,dirct in iolist[base][1]:
			if v == to:
				return dirct
		
		return None
	


	@classmethod
	def makeLinkList(self,iolist,linklist):

		for idx, val in enumerate(iolist):
			rtn = None
			for v,direct in val[0]:
				if isinstance(v, (int)):
					rtn = self.getLink(iolist,v,idx)
					if rtn != None:
						linklist.append([[rtn,v],[direct,idx]])
			

	
	@classmethod
	def graphFindList(self,mod,sumiobj,liststk):

		if mod in sumiobj:
			return
		
		sumiobj.add(mod)

		for k in mod._outkwd:
			for oobj in mod.outlist[k]:
				if isinstance(oobj,list):
					liststk.append(oobj)
			
		for key in mod.inplist.keys():
			for iobj in mod.inplist[key]:
				if isinstance(iobj,NysolMOD_CORE):
					self.graphFindList(iobj,sumiobj,liststk)
	
		return


	@classmethod
	def graphSetList(self,mod,sumiobj,liststk,pos):

		if mod in sumiobj:
			return pos
		
		sumiobj.add(mod)

		for k in mod._outkwd:
			for ipos in range(len(mod.outlist[k])):
				if isinstance(mod.outlist[k][ipos],list):
					mod.outlist[k][ipos] = liststk[pos]
					pos+=1
			
		for key in mod.inplist.keys():
			for iobj in mod.inplist[key]:
				if isinstance(iobj,NysolMOD_CORE):
					pos = self.graphSetList(iobj,sumiobj,liststk,pos)
	
		return pos

	@classmethod
	def makeRunNetworks(self,mods,useIter=False):

		# stock list obj before deepcopy
		listStks =[]
		sumiobj=set([])

		for mod in mods: 
			self.graphFindList(mod,sumiobj,listStks)

		dupobjs = copy.deepcopy(mods)

		# set list obj after deepcopy		
		sumiobj=set([])
		pos = 0
		for mod in dupobjs: 
			pos = self.graphSetList(mod,sumiobj,listStks,pos)

		# remove Unnecessary object
		if len(dupobjs) > 1:
			for dobj in dupobjs: 
				for k in dobj.outlist.keys():
					newlist = []
					for i,e in  enumerate(dobj.outlist[k]):
						if isinstance(e,NysolMOD_CORE):
							if dobj.checkRmObj(k,i,dupobjs):
								continue
						newlist.append(e)

					dobj.outlist[k].clear()
					dobj.outlist[k].extend(newlist)
						

		else:

			if dupobjs[0].disabled_ouputlist == False:
				for k in dupobjs[0].outlist.keys():
					# 不要 mod 除去
					newlist = [e for e in dupobjs[0].outlist[k] if not isinstance(e,NysolMOD_CORE)]
					dupobjs[0].outlist[k].clear()
					dupobjs[0].outlist[k].extend(newlist)


		runcnt=0

		for dobj in dupobjs:
			
			if dobj.disabled_ouputlist == True:
				runcnt+=1
				continue
			
			for k in dobj.outlist.keys():
				# 不要 mod 除去
				#newlist = [e for e in dobj.outlist[k] if not isinstance(e,NysolMOD_CORE)]
				#dobj.outlist[k].clear()
				#dobj.outlist[k].extend(newlist)

				#全てNysolMOD_COREか
				allNcore = True
				for ki in range(len(dobj.outlist[k])):
					if not isinstance(dobj.outlist[k][ki],NysolMOD_CORE):
						allNcore = False
						break

				if allNcore  and k == dobj.nowdir:
					runcnt+=1

				for e in dobj.outlist[k]:
					if not isinstance(e,NysolMOD_CORE) :
						runcnt+=1


		outfs   = [None]*len(dupobjs)
		runobjs = [None]*runcnt
		
		rpos = 0

		for i, dupobj in enumerate(dupobjs):

			 # 元 output object セット
			if True == dupobj.disabled_ouputlist or useIter:

				#  最終list不可はそのまま
				runobjs[rpos]= dupobj			
				rpos+=1
				outfs[i] = []

			elif dupobj.name == "writelist":

				if dupobj.okwdObjCnt() == 0:
	
					dupobj.outlist[dupobj.nowdir] =[list()]
						
				runobjs[rpos]= dupobj
				outfs[i] = runobjs[rpos].outlist[runobjs[i].nowdir][0]
				rpos+=1

			else:

				for k in dupobj.outlist.keys():

					#全てNysolMOD_COREか
					allNcore = True
					for ki in range(len(dupobj.outlist[k])):
						if not isinstance(dupobj.outlist[k][ki],NysolMOD_CORE):
							allNcore = False
							break

					if allNcore  and k == dupobj.nowdir:
						if "nfn" in dupobj.kwd and dupobj.kwd["nfn"] == True :
							runobjs[rpos]= dupobj.writelist(list(),sysadd=True,nfn=True)
						else:
							runobjs[rpos]= dupobj.writelist(list(),sysadd=True)
							
						outfs[i] = runobjs[rpos].outlist[runobjs[rpos].nowdir][0]
						rpos+=1
						continue

					for ki in range(len(dupobj.outlist[k])):

						if isinstance(dupobj.outlist[k][ki],list):

							from nysol.mcmd.submod.writelist import Nysol_Writelist as mwritelist
							wobj = mwritelist(dupobj.outlist[k][ki],sysadd=True)
							wobj.inplist[wobj.nowidir]=[dupobj]
							dupobj.outlist[k][ki] = wobj
							runobjs[rpos]= wobj

							if  k == dupobj.nowdir and not isinstance(dupobj.outlist[k][ki],NysolMOD_CORE):
								outfs[i] = runobjs[rpos].outlist[runobjs[rpos].nowdir][0]

							rpos += 1

						elif not isinstance(dupobj.outlist[k][ki],NysolMOD_CORE):

							runobjs[rpos]= dupobj
							
							if  k == dupobj.nowdir and not isinstance(dupobj.outlist[k][ki],NysolMOD_CORE):
								outfs[i] = runobjs[rpos].outlist[runobjs[rpos].nowdir][0]

							rpos += 1


		self.change_modNetworks(runobjs)
		
		uniqmod={} 
		sumiobj= set([])


		for mod in runobjs:
			mod.selectUniqMod(sumiobj,uniqmod)

		modlist=[None]*len(uniqmod) #[[name,para]]
		iolist=[None]*len(uniqmod) #[[iNo],[mNo],[oNo],[uNo]]

		self.makeModList(uniqmod,modlist,iolist)
		linklist=[]
		self.makeLinkList(iolist,linklist)
		return modlist,iolist,linklist,outfs

	@classmethod
	def runs(self,mods,**kw_args):

		msgF    = NysolMOD_CORE.OutMsg 
		modlimt = NysolMOD_CORE.RunLimit
		threxc  = False
		logDir = ""


		if "msg" in kw_args:
			if kw_args["msg"] == "on" :
				msgF = True

		if "dlog" in kw_args:
			if type(kw_args["dlog"]) is str and kw_args["dlog"] != "":
				import nysol.util as nu
				logDir = kw_args["dlog"]
				nu.mkDir(logDir)


		if "runlimit" in kw_args:
			modlimt = int(kw_args["runlimit"])

		if "throwexc" in kw_args:
			if kw_args["throwexc"] == True:
				threxc =True

		modlist,iolist,linklist,outfs = NysolMOD_CORE.makeRunNetworks(mods)

		py_msg=False
		try:
			if get_ipython().__class__.__name__ == 'ZMQInteractiveShell':
				py_msg = True
		except:
			pass

		kgpymsg = os.environ.get('KG_UsingPySysMsg')
		if kgpymsg != None: 
			py_msg = bool(int(kgpymsg))

		# 仮 kgshellへ移行
		import psutil as ps

		shobj = n_core.init(msgF,modlimt,int(ps.virtual_memory().total*0.9),py_msg,logDir)
		sts = n_core.runLx(shobj,modlist,linklist)
		if threxc and sts != 0 :
			raise Exception("run error")

		return outfs

	@classmethod
	def runsInnerForkTest(self,mods,**kw_args):

		pid = os.fork()	
		if pid == 0 : 

			msgF    = NysolMOD_CORE.OutMsg 
			modlimt = NysolMOD_CORE.RunLimit
			threxc  = False
			logDir = ""

			if "msg" in kw_args:
				if kw_args["msg"] == "on" :
					msgF = True

			if "dlog" in kw_args:
				if type(kw_args["dlog"]) is str and kw_args["dlog"] != "":
					import nysol.util as nu
					logDir = kw_args["dlog"]
					nu.mkDir(logDir)


			if "runlimit" in kw_args:
				modlimt = int(kw_args["runlimit"])

			if "throwexc" in kw_args:
				if kw_args["throwexc"] == True:
					threxc =True

			modlist,iolist,linklist,outfs = NysolMOD_CORE.makeRunNetworks(mods)

			py_msg=False
			try:
				if get_ipython().__class__.__name__ == 'ZMQInteractiveShell':
					py_msg = True
			except:
				pass

			kgpymsg = os.environ.get('KG_UsingPySysMsg')
			if kgpymsg != None: 
				py_msg = bool(int(kgpymsg))

			# 仮 kgshellへ移行
			import psutil as ps
			shobj = n_core.init(msgF,modlimt,int(ps.virtual_memory().total*0.9),py_msg,logDir)
			sts = n_core.runLx(shobj,modlist,linklist)
			if threxc and sts != 0 :
				raise Exception("run error")
			return outfs
		else:
			os.waitpid(pid, 0)
			sys.exit()
		
	@classmethod
	def drawModels(self,mod,fname=None):

		modlist,iolist,linklist,_ = NysolMOD_CORE.makeRunNetworks(mod)
		ndraw.chageSVG(modlist,iolist,linklist,fname)		

	@classmethod
	def drawModelsD3(self,mod,fname=None):

		modlist,iolist,linklist,x = NysolMOD_CORE.makeRunNetworks(mod)
		ndraw.chageSVG_D3(modlist,iolist,linklist,fname)



	#GRAPH表示 #deepコピーしてからチェック
	@classmethod
	def modelInfos(self,mod):

		modlist,iolist,linklist,_ = NysolMOD_CORE.makeRunNetworks(mod)
		return {"modlist":modlist,"iolist":iolist,"linklist":linklist}

	@classmethod
	def makeScps(self,mod,fname=None,**kw_args):

		msgF = False
		if "msg" in kw_args:
			if kw_args["msg"] == "on" :
				msgF = True


		modlist,iolist,linklist,endobj = NysolMOD_CORE.makeRunNetworks(mod)
		ndraw.chageSCP(modlist,iolist,linklist,fname,msgF)
		#ndraw.chageSCP(modlist,iolist,linklist,endobj,fname)




	def run(self,**kw_args):

		return NysolMOD_CORE.runs([self],**kw_args)[0]


	def makeRunNetwork(self,useIter=False):

		return NysolMOD_CORE.makeRunNetworks([self],useIter)


	def drawModel(self,fname=None):

		NysolMOD_CORE.drawModels([self],fname)


	def drawModelD3(self,fname=None):

		NysolMOD_CORE.drawModelsD3([self],fname)


	def modelInfo(self):

		return NysolMOD_CORE.modelInfos([self])

	def makeScp(self,fname=None,**kw_args):

		return NysolMOD_CORE.makeScps([self],fname,**kw_args)


	def getMsgFlg(self):
		return NysolMOD_CORE.OutMsg

	def modelcopy(self):
		# stock list obj before deepcopy
		listStks =[]
		sumiobj=set([])

		self.graphFindList(self,sumiobj,listStks)

		dupobj = copy.deepcopy(self)

		# set list obj after deepcopy		
		sumiobj=set([])
		self.graphSetList(dupobj,sumiobj,listStks,0)
		return dupobj


def runs(val,**kw_args):
	return NysolMOD_CORE.runs(val,**kw_args)

def drawModels(val,fname):
	return NysolMOD_CORE.drawModels(val,fname)

def drawModelsD3(val,fname):
	return NysolMOD_CORE.drawModelsD3(val,fname)

def modelInfos(val):
	return NysolMOD_CORE.modelInfos(val)


def makeScps(val,fname,**kw_args):
	return NysolMOD_CORE.makeScps(val,fname,**kw_args)


def setMsgFlg(val):
	NysolMOD_CORE.setMsgFlg(val)

def setRunLimit(val):
	NysolMOD_CORE.setRunLimit(val)




# -*- coding: utf-8 -*-
import nysol._nysolshell_core as n_core
from nysol.mcmd.nysollib.core import NysolMOD_CORE
from nysol.mcmd.nysollib import nysolutil as nutil

class Nysol_Writedict(NysolMOD_CORE):

	_kwd ,_inkwd,_outkwd  = n_core.getparalist("writelist",3)
	_kwd[0].append("dtype")
	_kwd[1].append("dict")

	def __init__(self,*args, **kw_args) :
		kw_args["dict"] = True
		super(Nysol_Writedict,self).__init__("writelist",nutil.args2dict(args,kw_args,Nysol_Writedict._kwd,uk="o"))


def writedict(self,*args, **kw_args):
	kw_args["dict"] = True
	return Nysol_Writedict(nutil.args2dict(args,kw_args,Nysol_Writedict._kwd,uk="o")).addPre(self)

setattr(NysolMOD_CORE, "writedict", writedict)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import shutil
import os
import nysol.mcmd as nm
import nysol.util as nu

#import nysol.util.mtemp as mtemp
#import nysol.util.mrecount as mrecount
from nysol.take.lib import items as items
#=トランザクションデータクラス
# 頻出パターンマイニングで使われるトランザクションデータを扱うクラス。
# トランザクションファイルは、トランザクションID項目とアイテム集合項目から構成される。
# アイテム集合は、スペースで区切られた文字列集合として表現される。
#
#===利用例
# 以下tra.csvの内容
# ---
# items
# a b d
# a d e
# c e
# a d
# c d e
# c e
# a d e
#
# 以下taxo.csvの内容
# ---
# item,taxo
# a,X1
# b,X1
# c,X2
# d,X2
# e,X2
#
# 以下rubyスクリプト
# ---
# require 'rubygems'
# require 'mining'
#
# tra=TraDB.new("tra.csv",nil,"items")
# puts tra.file       # => ./1252813069_2179/dat/1
# puts tra.numTraFile # => nil
# puts tra.idFN       # => tid
# puts tra.itemsetFN  # => items
# puts tra.recCnt     # => 7
# p    tra.items      # => #<Items:0x16a39ec @iItemFN=["items"],...
#
# taxo=Taxonomy.new("taxo.csv","item","taxo")
# tra.addTaxo(taxo)
# tra.mkNumTra
# puts tra.numTraFile #=> ./1252813069_2179/dat/6
# p    tra.items      #=> #<Items:0x16a39ec @iItemFN=["items", "taxo"],..., @taxonomy=#<Taxonomy:0x168ba7c ...
#
#=====./1252813069_2179/dat/1 の内容
# tid,items
# 1,a b d
# 2,a d e
# 3,c e
# 4,a d
# 5,c d e
# 6,c e
# 7,a d e
#
#=====./1252813069_2179/dat/6 の内容
# 1 2 3 4 5 6 7
# 1 2 4 6 7
# 1 4 5 6 7
# 3 5 7
# 1 4 6 7
# 3 4 5 7
# 3 5 7
# 1 4 5 6 7
#
class TraDB(object):


	#=== TraDBクラスの初期化
	#
	#====引数
	# iFile: transactionファイル名
	# iItemsetFN: iFile上のアイテム集合項目名
	# idFN: 新しく命名するトランザクションID項目名
	# itemsetFN: 新しく命名するアイテム集合項目名
	#
	#====機能
	#* トランザクションIDとアイテム集合の2項目から構成するトランザクションデータを新たに作成する。
	#* ID項目が指定されていなければ、1から始まる連番によってID項目を新規に作成する。
	#* トランザクション件数(iFileのレコード件数)を計算する。
	#* アイテム集合項目からItemsオブジェクトを生成する。
	def __init__(self,iFile,idFN,itemFN,clsFN=None):
		self.file     = None # トランザクションファイル名
		self.idFN     = None # トランザクションID項目名(String)
		self.itemFN   = None # アイテム集合項目名(String)
		self.clsFN    = None # クラス項目名(String)
		self.traSize  = None # トランザクションサイズ(Num)
		self.items    = None # Itemsクラス
		self.taxonomy = None # 階層分類クラス
		self.clsNameRecSize = None # クラス別件数
		self.clsSize  = None # クラス数
		self.cFile    = None # クラスファイル

		self.temp   = nu.Mtemp()
		self.iFile  = iFile                    # 入力ファイル
		self.iPath  = os.path.abspath(self.iFile) # フルパス
		self.idFN   = idFN                     # トランザクションID項目名
		self.itemFN = itemFN                   # 入力ファイル上のアイテム集合項目名
		self.file   = self.temp.file()               # 出力ファイル名
		
		fpara = self.idFN + "," + self.itemFN

		nm.mcut(f=fpara,i=self.iFile).muniq(k=fpara,o=self.file).run()

		# レコード数の計算
		self.recCnt = nu.mrecount(i=self.file)

		# トランザクション数の計算
		xx1 = nm.mcut(f=self.idFN,i=self.file).muniq(k=self.idFN).mcount(a='__cnt').mcut(f='__cnt').run()
		self.traSize = int(xx1[0][0])
		
		# トランザクションデータからアイテムオブジェクトを生成
		self.items=items.Items(self.file,self.itemFN)

		# クラスデータ
		if clsFN :
			self.clsFN=clsFN
			self.cFile=self.temp.file()
			fpara_c = "%s,%s"%(self.idFN,self.clsFN)

			nm.mcut(f=fpara_c,i=self.iFile).muniq(k=fpara_c,o=self.cFile).run()

			# 文字列としてのクラス別件数配列を数値配列に変換する
			self.clsSize=0
			self.clsNames = []
			self.clsNameRecSize = {}
			for vv in nm.mcut(f=self.clsFN,i=self.cFile).mcount(k=self.clsFN,a='count'):
				self.clsNames.append(vv[0])	 
				self.clsNameRecSize[vv[0]] = int(vv[1]) 
				self.clsSize+=1


	def replaceFile(self,train):

		self.file   = train

		# レコード数の計算
		self.recCnt = nu.mrecount(i=self.file)


		xx1 = nm.mcut(f=self.idFN,i=self.file).muniq(k=self.idFN).mcount(a='__cnt').mcut(a='__cnt').run()
		self.traSize = int(xx1[0][0])

	#=== taxonomyをトランザクションに追加
	# トランザクションデータのアイテム集合に、対応するtaxonomyを追加する。
	#
	#====引数
	# taxonomy: Taxonomyオブジェクト。
	#
	#====機能
	#* トランザクションデータのアイテム集合項目におけるアイテム全てについて、対応するtaxonomyをアイテム集合として追加する。
	def addTaxo(self,taxonomy):

		self.taxonomy=taxonomy

		self.items.addTaxo(self.taxonomy) # アイテムクラスにtaxonomyを追加する

		tFile =taxonomy.file
		itemFN=taxonomy.itemFN
		taxoFN=taxonomy.taxoFN

		tf=nu.Mtemp()
		xx1=tf.file()
		cpara = "%s,%s:%s"%(self.idFN,taxoFN,self.itemFN)
		f=None
		f <<= nm.mnjoin(i=self.file,k=self.itemFN,K=itemFN,f=taxoFN,m=tFile)
		f <<= nm.mcut(f=cpara,o=xx1)
		f.run()


		ipara ="%s,%s"%(self.file,xx1)
		kpara ="%s,%s"%(self.idFN,self.itemFN)
		xx2=tf.file()
		nm.mcat(i=ipara).muniq(k=kpara,o=xx2).run(msg="on")

		self.file = self.temp.file()
		shutil.move(xx2,self.file)



	def repTaxo(self,taxonomy):

		#@taxonomy=taxonomy #replaceの場合はtaxonomyを登録しない

		self.items.repTaxo(taxonomy) # アイテムクラスをtaxonomyで置換する

		tFile =taxonomy.file
		itemFN=taxonomy.itemFN
		taxoFN=taxonomy.taxoFN

		tf=nu.Mtemp()
		xx1=tf.file()
		cpara = "%s,%s:%s"%(self.idFN,taxoFN,self.itemFN)
		kpara = "%s,%s"%(self.idFN,self.itemFN)
		
		nm.mjoin(i=self.file,k=self.itemFN,K=itemFN,f=taxoFN,m=tFile).mcut(f=cpara).muniq(k=kpara,o=xx1).run()

		self.file = self.temp.file()
		shutil.move(xx1,self.file)




# -*- coding: utf-8 -*-
import shlex
import re

def para2strStr(vals):
	rtnStr = ""
	for k, v in vals.items():

		if isinstance(v,bool) :
			if v==True:
				rtnStr += "-" + k + " "

		elif isinstance(v,str) :
			rtnStr += k + "=" + v + " "

		elif isinstance(v,float) :
			rtnStr += k + "=" + str(v) + " "

		elif isinstance(v,int) :
			rtnStr += k + "=" + str(v) + " "

		elif isinstance(v,list) :
			plist = []
			for val in v:
				if isinstance(val,str) :
					plist.append(val)
				elif isinstance(val,float) or isinstance(v,int) :
					plist.append(str(val))
			rtnStr += k + "=" + ",".join(plist) + " "

		elif isinstance(v,dict) :

			plist = []
			for vk, vv in v.items():
				v1=""
				if isinstance(vk,str) :
					v1 = vk
				elif isinstance(vk,float) or isinstance(vk,int) :
					v1 = str(vk)
				v2=""
				if isinstance(vv,str) :
					v2 = vv
				elif isinstance(vv,float) or isinstance(vv,int) :
					v2 = str(vk)
				plist.append(v1+":" +v2)
			rtnStr += k + "=" + ",".join(plist) + " "

	return rtnStr


def para2str(vals): #クラスごとにいれる?

	rtnStr = []
	for k, v in vals.items():

		#特殊パラメータ　runfunc_*
		if k == "runfunc_func" or k == "runfunc_args" or k == "runfunc_kwd" :
			rtnStr.append([k,v])

		elif isinstance(v,bool) :
			if v==True:
				rtnStr.append("-" + k )

		elif isinstance(v,str) :
			rtnStr.append( k+"="+v)

		elif isinstance(v,float) :
			rtnStr.append(k + "=" + str(v))

		elif isinstance(v,int) :
			rtnStr.append(k + "=" + str(v) )

		elif isinstance(v,list) :
			plist = []
			for val in v:
				if isinstance(val,str) :
					plist.append(val)
				elif isinstance(val,float) or isinstance(v,int) :
					plist.append(str(val))

			rtnStr.append(k + "=" + ",".join(plist) )

		elif isinstance(v,dict) :

			plist = []
			for vk, vv in v.items():
				v1=""
				if isinstance(vk,str) :
					v1 = vk
				elif isinstance(vk,float) or isinstance(vk,int) :
					v1 = str(vk)
				v2=""
				if isinstance(vv,str) :
					v2 = vv
				elif isinstance(vv,float) or isinstance(vv,int) :
					v2 = str(vk)
				plist.append(v1+":" +v2)

			rtnStr.append(k + "=" + ",".join(plist) )

		else:
			rtnStr.append([k,v])
							
	return rtnStr



def args2dict(args, kw_args,klist,uk=None):

	if len(args)>1:
		raise TypeError("arge only one")
		return None

	if len(args)==1:
		if isinstance(args[0],str) :
			for para in shlex.split(args[0]):
				val = para.split("=",1)
				if len(val)==1 :
					if val[0][0] == '-':
						kw_args[re.sub(r'^-', "", val[0])] = True
					elif uk!=None:
						kw_args[uk] = val[0]
					else:
						raise TypeError("arge only one")
				elif len(val)==2 :
					kw_args[val[0]] = val[1]
				else:
					print("unkonwn parameter")
		elif isinstance(args[0],dict):
			kw_args.update(args[0])
		elif isinstance(args[0],list) and uk!=None:
			kw_args[uk] = args[0]
		elif isinstance(args[0],tuple):
			pass

		else :
			raise TypeError("args type str or dict")
			#print(args)
			#print("args type str or dict")
			#return None

	# check parameter
	exval = []
	for k,v in kw_args.items():
		if k in klist[0] and isinstance(v,str):
			pass
		elif k in klist[0] :
			pass
		elif k in klist[1] :
			if v== True:
				pass
			else:
				exval.append(k)
		elif k == "tag" or k == "dlog" or k == "sysadd" :
			pass

		else:
			exval.append(k)
			wxc = TypeError(k + " is not keyword")
			raise (wxc)
			
	for k in exval:
		del kw_args[k]

	return kw_args


# あるキーワードが入ってるかチェック
def iskwd(args, kw_args,kwd):
	if len(args)>1:
		print("args only one")
		return False

	if len(args)==1:
		if isinstance(args[0],dict):
			if kwd in args[0]:
				return True
	
	if kwd in kw_args:
		return True



def arg2dict(args, kw_args,klist,uk=None):

	if len(args)>1:
		print("args only one")
		return None

	if len(args)==1:
		if isinstance(args[0],str) :
			#kw_args["cmdstr"] = "'" + args[0] + "'" 
			kw_args["cmdstr"] = args[0]

		elif isinstance(args[0],dict):
			kw_args.update(args[0])
		elif isinstance(args[0],tuple):
			pass

		else :
			raise TypeError
			#print(args)
			#print("args type str or dict")
			#return None

	# check parameter
	exval = []
	for k,v in kw_args.items():
		if k in klist[0] and isinstance(v,str):
			pass
		elif k in klist[0] :
			pass
		elif k in klist[1] and v== True:
			pass
		elif k == "tag" or k == "dlog" :
			pass
		else:
			exval.append(k)
			print ( k + " is not keyword")
			
	for k in exval:
		del kw_args[k]

	return kw_args






#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil

import nysol.mcmd as nm
import nysol.util as nu
import nysol.util.margs as margs
import nysol.util.mtemp as mtemp
from nysol.take import extcore as extTake


class mbiclique(object):

	helpMSG="""
----------------------------
mbiclique.rb version #{$version}
----------------------------
概要) lcmによる極大二部クリークの列挙
内容) 二部グラフデータを入力として、極大二部クリークを列挙する。
書式) mbiclique.rb ei= [ef=] [o=] [l=] [u=] [o=] [-edge] [T=] [-debug] [--help]

  ファイル名指定
  ei=    : 辺データファイル
  ef=    : 辺データ上の2つの部項目名(省略時は"node1,node2")
  o=     : 出力ファイル
  l=     : 二部クリークを構成する最小節点数(ここで指定したサイズより小さいクリークは列挙されない)
         : カンマで区切って2つの値を指定すると、各部のサイズを制限できる
         : 1つ目の値はef=で指定した1つ目の部に対応し、2つ目の値は2つ目に指定した部に対応する。
  u=     : クリークを構成する最大節点数(ここで指定したサイズより大きいクリークは列挙されない)
         : カンマで区切って2つの値を指定すると、各部のサイズを制限できる
	-edge  : 枝による出力(クリークIDと枝(節点ペア)で出力する)

  その他
  T= : ワークディレクトリ(default:/tmp)
  --help : ヘルプの表示

入力形式)
二部グラフの節点ペアを項目で表現したCSVデータ。

出力形式1)
二部クリークを構成する全節点を各部ごとにベクトル形式で出力する。
出力項目は、"節点項目名1,節点項目名2,size1,size2"の4項目で、節点名1と節点名2は、ef=で指定された名称が利用される。
節点項目名1,節点項目名2に出力される値が節点名ベクトルである(一行が一つの二部クリークに対応)ことが異なる。
idはクリークの識別番号で、一つのクリークは同じid番号が振られる。id番号そのものに意味はない。
節点項目名1,節点項目名2には、各部を構成する節点名のベクトルが出力される。
size1,size2は二部クリークを構成する各部の節点数である。

出力形式2) -edge を指定した場合の出力形式
クリークIDと二部クリークを構成する全枝(節点ペア)を出力する。
出力項目は"id,節点項目名1,節点項目名2,size"の4項目である。
例えば各部のサイズが3,4であるような二部クリークは12行の枝データとして出力される。
出力形式1に比べてファイルサイズは大きくなる。


例1)
$ cat data1.csv
node1,node2
a,A
a,B
a,C
b,A
b,B
b,D
c,A
c,D
d,B
d,C
d,D

$ mclique.rb ei=data1.csv ef=n1,n2 o=out1.csv
#MSG# converting paired form into transaction form ...; 2014/03/24 11:52:05
#MSG# lcm_20140215 CIf /tmp/__MTEMP_47150_70177387663280_0 1 /tmp/__MTEMP_47150_70177387663280_3; 2014/03/24 11:52:05
trsact: /tmp/__MTEMP_47150_70177387663280_0 ,#transactions 4 ,#items 4 ,size 11 extracted database: #transactions 4 ,#items 4 ,size 11
output to: /tmp/__MTEMP_47150_70177387663280_3
separated at 0
11
1
3
4
3
iters=11
#END# mbiclique.rb ei=data1.csv o=out1.csv ef=node1,node2
$ cat out1.csv 
node1,node2,size1,size2
a,A B C,1,3
a b,A B,2,2
a b c,A,3,1
a b d,B,3,1
a d,B C,2,2
b,A B D,1,3
b c,A D,2,2
b c d,D,3,1
b d,B D,2,2
d,B C D,1,3

例3) 枝による出力(-edgeの指定)
$ mclique.rb ei=data1.csv ef=n1,n2 o=out2.csv
#END# ../../bin/mbiclique.rb ei=data/data1.csv o=xxresult/out11.csv ef=node1,node2 -edge
$ cat out2.csv 
id,node1,node2,size1,size2
1,c,A,3,1
1,a,A,3,1
1,b,A,3,1
10,d,B,1,3
10,d,C,1,3
10,d,D,1,3
2,b,B,3,1
2,a,B,3,1
2,d,B,3,1
   :

例3) 部node1の最小サイズを3に制限
$ mbiclique.rb ei=data1.csv o=out3.csv ef=node1,node2 l=3,
#END# mbiclique.rb ei=data1.csv o=out3.csv ef=node1,node2 l=3,
$ cat out3.csv 
node1,node2,size1,size2
a b c,A,3,1
a b d,B,3,1
b c d,D,3,1

# Copyright(c) NYSOL 2012- All Rights Reserved.
"""

	verInfo="version=1.2"

	paramter = {	
		"ei":"filename",
		"ef":"fld",
		"l":"int",
		"u":"int",
		"o":"filename",		
		"edge":"bool",
		"T":"str"
	}

	paramcond = {	
		"hissu": ["ei"]
	}	


	def help():
		print(mbiclique.helpMSG) 

	def ver():
		print(mbiclique.verInfo)

	def __param_check_set(self , kwd):

		for k,v in kwd.items():
			if not k in mbiclique.paramter	:
				raise( Exception("KeyError: {} in {} ".format(k,self.__class__.__name__) ) )
			#型チェック入れる

		self.msgoff = True

		self.byedge  = kwd["edge"] if "edge" in kwd else False 
		self.oFile   = kwd["o"]   if "o"   in kwd else None

		# ---- edge field names (two nodes) on ei=
		self.ei = kwd["ei"] # edgeファイル名
		ef0 = kwd["ef"].split(",")
		self.ef1 = ef0[0]
		self.ef2 = ef0[1] 

		minSize = kwd["l"].split(",") if "l"   in kwd else None   # クリークサイズ下限
		maxSize = kwd["u"].split(",") if "u"   in kwd else None   # クリークサイズ上限

		self.minSize1 = ""
		self.minSize2 = ""
		self.maxSize1 = ""
		self.maxSize2 = ""


		if minSize != None :

			if len(minSize) == 1:
				if minSize[0] != "":
					self.minSize1 = minSize[0]
					self.minSize2 = minSize[0]
			else:
				if minSize[0] != "":
					self.minSize1 = minSize[0]
				if minSize[1] != "":
					self.minSize2 = minSize[1]

		if maxSize != None :

			if len(maxSize) == 1:
				if maxSize[0] != "":
					self.maxSize1 = maxSize[0]
					self.maxSize2 = maxSize[0]
			else:
				if maxSize[0] != "":
					self.maxSize1 = maxSize[0]
				if maxSize[1] != "":
					self.maxSize2 = maxSize[1]

	def __cmdline(self):
		cmdline = self.__class__.__name__
		for k,v in self.args.items():
			if type(v) is bool :
				if v == True :
					cmdline += " -" + str(k)
			else:
				cmdline += " " + str(k) + "=" + str(v)
		return cmdline 

	def __init__(self,**kwd):
		#パラメータチェック
		self.args = kwd
		self.__param_check_set(kwd)



	def pair2tra(self,ei,ef1,ef2,traFile,mapFile1,mapFile2):

		f1 = None
		f1 <<= nm.mcut(f="{}:node1".format(self.ef1),i=self.ei)
		f1 <<= nm.mdelnull(f="node1")
		f1 <<= nm.muniq(k="node1")
		f1 <<= nm.mnumber(s="node1", a="num1",o=mapFile1)

		f2 = None
		f2 <<= nm.mcut(f="{}:node2".format(self.ef2),i=self.ei)
		f2 <<= nm.mdelnull(f="node2")
		f2 <<= nm.muniq(k="node2")
		f2 <<= nm.mnumber(s="node2",a="num2",o=mapFile2)

		f3 = None
		f3 <<= nm.mcut(f="{}:node1,{}:node2".format(self.ef1,self.ef2),i=self.ei)
		f3 <<= nm.mjoin(k="node1",m=f1,f="num1")
		f3 <<= nm.mjoin(k="node2",m=f2,f="num2")
		f3 <<= nm.mcut(f="num1,num2")
		f3 <<= nm.msortf(f="num1,num2%n")
		f3 <<= nm.mtra(k="num1",s="num2%n",f="num2")
		f3 <<= nm.msortf(f="num1%n")
		f3 <<= nm.mcut(f="num2",nfno=True,o=traFile)
		f3.run()
		os.system("cat "+traFile)

	# ============
	# entry point
	def run(self,**kw_args):

		os.environ['KG_ScpVerboseLevel'] = "2"
		if "msg" in kw_args:
			if kw_args["msg"] == "on":
				os.environ['KG_ScpVerboseLevel'] = "4"

		tempW	= mtemp.Mtemp()

		xxtra = tempW.file()
		xxmap1 = tempW.file()
		xxmap2 = tempW.file()
		lcmout = tempW.file()

		xxt0 = tempW.file()
		xxp0 = tempW.file()
		xx3t = tempW.file()
		xx4t = tempW.file()

		self.pair2tra(self.ei,self.ef1,self.ef2,xxtra,xxmap1,xxmap2)

		runPara={}
		runPara["type"] = "CIf"
		runPara["sup"] = 1
		runPara["o"] = lcmout
		runPara["i"] = xxtra

		if self.minSize2:
			runPara["l"] = self.minSize2
		if self.maxSize2:
			runPara["u"] = self.maxSize2


		extTake.lcm(runPara)
		extTake.lcmtrans(lcmout,"p",xxt0)

		f = None
		f <<= nm.mdelnull(f="pattern",i=xxt0)
		f <<= nm.mvreplace(vf="pattern",m=xxmap2,K="num2",f="node2")
		f <<= nm.mcut(f="pid,pattern,size:size2")
		f <<= nm.mvsort(vf="pattern")
		f <<= nm.msortf(f="pid")
		
		if self.byedge :
			f_e0 = nm.mtra(f="pattern",i=f,r=True)
			extTake.lcmtrans(lcmout,"t",xx3t)

			f_e1 = None
			f_e1 <<= nm.mjoin(k="__tid",m=xxmap1,f="node1",K="num1",i=xx3t)
			f_e1 <<= nm.msortf(f="pid")
			## xx2
			f_e2 = None
			f_e2 <<= nm.mcount(k="pid",a="size1",i=f_e1)
			f_e2 <<= nm.mselnum(f="size1",c="[{},{}]".format(self.minSize1,self.maxSize1))
			
			f_e3 = None
			f_e3 <<= nm.mjoin(k="pid",m=f_e2,f="size1",i=f_e1)
			f_e3 <<= nm.mnjoin(k="pid",m=f_e0,f="pattern,size2")
			f_e3 <<= nm.mcut(f="pid:id,node1:{},pattern:{},size1,size2".format(self.ef1,self.ef2),o=self.oFile)
			f_e3.run()

		else:

			extTake.lcmtrans(lcmout,"t",xx4t)
			f_e4 = None
			f_e4 <<= nm.mjoin(k="__tid",m=xxmap1,i=xx4t,f="node1",K="num1")
			f_e4 <<= nm.mtra(k="pid",f="node1")
			f_e4 <<= nm.mvcount(vf="node1:size1")
			f_e4 <<= nm.mjoin(k="pid",m=f,f="pattern,size2")
			f_e4 <<= nm.mselnum(f="size1",c="[{},{}]".format(self.minSize1,self.maxSize1))
			f_e4 <<= nm.mvsort(vf="node1,pattern")
			f_e4 <<= nm.msortf(f="node1,pattern")
			f_e4 <<= nm.mcut(f="node1:{},pattern:{},size1,size2".format(self.ef1,self.ef2),o=self.oFile)
			f_e4.run()

		nu.mmsg.endLog(self.__cmdline())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import os.path
import nysol.mcmd as nm
import nysol.util as nu

import nysol.util.margs as margs
from nysol.take.lib.base import traDB as tdb
from nysol.take.lib import taxonomy as taxolib
from nysol.take.lib import enumLcmIs as elcmIs
from nysol.take.lib import enumLcmEp as elcmEp


class mitemset(object):

	helpMSG="""
----------------------------
#{$cmd} version #{$version}
----------------------------
概要) LCMにより多頻度アイテム集合を列挙する
特徴) 1) 分類階層を扱うことが可能
      2) 頻出パターン, 飽和頻出パターン, 極大頻出パターンの３種類のパターンを列挙可能
      3) クラスを指定することで、上記3パターンに関する顕在パターン(emerging patterns)を列挙可能
書式) #{$cmd} i= [x=] [O=] [tid=] [item=] [taxo=] [class=] [type=] [s=|S=] [sx=|Sx=] [l=] [u=] [p=] [top=] [-replaceTaxo] [T=] [--help]

例) #{$cmd} i=basket.csv tid=traID item=商品

  ファイル名指定
  i= : アイテム集合データベースファイル名【必須】
  x= : taxonomyファイル名【オブション】*1
  O= : 出力ディレクトリ名【オプション:default:./take_現在日付時刻】

  項目名指定
  tid=   : トランザクションID項目名(i=上の項目名)【オプション:default="tid"】
  item=  : アイテム項目名(i=,t=上の項目名)【オプション:default="item"】
  class= : クラス項目名(i=上の項目名)【オプション:default="class"】
  taxo=  : 分類項目名を指定する(x=上の項目名)【条件付き必須:x=】

  列挙パラメータ
  type= : 抽出するパターンの型【オプション:default:F, F:頻出集合, C:飽和集合, M:極大集合】
  s=    : 最小支持度(全トランザクション数に対する割合による指定)【オプション:default:0.05, 0以上1以下の実数】
  S=    : 最小支持度(件数による指定)【オプション】
  sx=   : 最大支持度(support)【オプション:default:1.0, 0以上1以下の実数】
  Sx=   : 最大支持件数【オプション】
  l=    : パターンサイズの下限(1以上20以下の整数)【オプション:default:制限なし】
  u=    : パターンサイズの上限(1以上20以下の整数)【オプション:default:制限なし】
  p=    : 最小事後確率【オプション:default:0.5】
  g=    : 最小増加率【オプション】
  top=  : 列挙するパターン数の上限【オプション:default:制限なし】*2

  その他
  -replaceTaxo : taxonomyを置換する
  T= : ワークディレクトリ(default:/tmp)
  --help : ヘルプの表示

  注釈
  *1 x=が指定されたとき、itemに対応するtaxonomyをトランザクションに追加して実行する。例えば、アイテムa,bのtaxonomyをX、c,dのtaxonomyをYとすると、あるトランザクションabdはabdXYとなる。
     ただし-replaceTaxoが指定されると、taxonomyは追加ではなく置換して実行する。前例ではトランザクションabdはXYに置換される。
  *2 top=が指定された時の動作: 例えばtop=10と指定すると、支持度が10番目高いパターンの支持度を最小支持度として頻出パターンを列挙する。よって、同じ支持度のパターンが複数個ある場合は10個以上のパターンが列挙されるかもしれない。

# より詳しい情報源 http://www.nysol.jp
# LCMの詳しい情報源 http://research.nii.ac.jp/~uno/codes-j.htm
# Copyright(c) NYSOL 2012- All Rights Reserved.
		"""

	verInfo="version=1.2"

	paramter = {	
		"i": "filename",
		"x": "fileName",
		"O": "str",
		"tid": "fld",
		"item":"fld",
		"cls" :"fld",
		"taxo":"fld",
		"type":"str",
		"s" : "float",
		"p": "float",
		"uniform" : "bool",
		"replaceTaxo": "bool",
		"S":"int",
		"sx":"float",
		"Sx":"int",
		"g":"float",
		"l":"int",
		"u":"int",
		"top":"int",
		"mcmdenv" : "bool",
		"T": "str"
	}
	paramcond = {	
		"hissu":"i"
	}
	
	def help():
		print(mitemset.helpMSG) 

	def ver():
		print(mitemset.versionInfo)

	
	def __param_check_set(self , kwd):

		for k,v in kwd.items():
			if not k in mitemset.paramter	:
				raise( Exception("KeyError: {} in {} ".format(k,self.__class__.__name__) ) )
			#型チェック入れる
			
		import datetime
		t = datetime.datetime.today()

		self.iFile   = kwd["i"]
		self.xFile   = kwd["x"]    if "x"    in kwd else None
		self.idFN    = kwd["tid"]  if "tid"  in kwd else "tid"
		self.itemFN  = kwd["item"] if "item" in kwd else "item"
		self.clsFN   = kwd["cls"]  if "cls"  in kwd else None

		self.taxoFN  = kwd["taxo"] if "taxo" in kwd else "taxo"
		self.outPath = kwd["O"]    if "O"    in kwd else "./take_{}".format(t.strftime("%Y%m%d%H%M%S"))
		
		self.eArgs={}
		self.eArgs["nomodel"] = True

		self.eArgs["type"   ] = kwd["type"]     if "type"    in kwd else "F"
		self.eArgs["minSup" ] = float(kwd["s"]) if "s"       in kwd else 0.05  # 0-1
		self.eArgs["minProb"] = float(kwd["p"]) if "p"       in kwd else 0.5   # 0.5-1
		self.eArgs["uniform"] = kwd["uniform"]  if "uniform" in kwd else False

		if "S" in kwd :
			self.eArgs["minCnt"] = int(kwd["S"])

		if "sx" in kwd :
			self.eArgs["maxSup"] = float(kwd["sx"]) # 0-1

		if "Sx" in kwd :
			self.eArgs["maxCnt"] = int(kwd["Sx"]) # 1-

		if "g" in kwd :
			self.eArgs["minGR"] = float(kwd["g"]) # 1.0

		if "l" in kwd :
			self.eArgs["minLen"] = int(kwd["l"]) # 1-

		if "u" in kwd :
			self.eArgs["maxLen"] = int(kwd["u"]) # 1-

		if "top" in kwd :
			self.eArgs["top"] = int(kwd["top"])


		self.replaceTaxo = kwd["replaceTaxo"] if "replaceTaxo" in kwd else False


	def __cmdline(self):
		cmdline = self.__class__.__name__
		for k,v in self.args.items():
			if type(v) is bool :
				if v == True :
					cmdline += " -" + str(k)
			else:
				cmdline += " " + str(k) + "=" + str(v)
		return cmdline 

	def __init__(self,**kwd):
		#パラメータチェック
		self.args = kwd
		self.__param_check_set(kwd)


	def run(self,**kw_args):

		os.environ['KG_ScpVerboseLevel'] = "2"
		if "msg" in kw_args:
			if kw_args["msg"] == "on":
				os.environ['KG_ScpVerboseLevel'] = "4"

		# V型DBの読み込み
		db = tdb.TraDB(self.iFile,self.idFN,self.itemFN,self.clsFN)
		# taxonomyのセット
		self.taxo=None
		if self.xFile!=None :
			self.taxo = taxolib.Taxonomy(self.xFile,self.itemFN,self.taxoFN)
			if self.replaceTaxo :
				db.repTaxo(self.taxo) # taxonomyの置換
			else:
				db.addTaxo(self.taxo) # taxonomyの追加

		# パターン列挙
		lcm=None
		if self.clsFN :
			lcm=elcmEp.LcmEp(db)
			lcm.enumerate(self.eArgs)
		else:
			lcm=elcmIs.LcmIs(db);
			lcm.enumerate(self.eArgs)
		# 出力
		if not os.path.isdir(self.outPath) :
			os.makedirs(self.outPath)
		lcm.output(self.outPath)

		nu.mmsg.endLog(self.__cmdline())
